SystemOrganization addCategory: #'QwaqDemo-Forums'!
SystemOrganization addCategory: #'QwaqDemo-Forums-Campus'!
SystemOrganization addCategory: #'QwaqDemo-Forums-Templates'!
SystemOrganization addCategory: #'QwaqDemo-Forums-Widgets'!

TConic subclass: #QTreeBase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QTreeBase commentStamp: '<historical>' prior: 0!
An admittedly rather specialized widget: a very flat cone
textured with brown ground material, 
used as a psuedo-shadow or root-bump  for campus trees.!

!QTreeBase class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QTreeBase methodsFor: 'initialize' stamp: 'bgf 9/22/2007 19:08'!
initialize
	super initialize.

	self material: (QMaterials groundMaterial: 'lawn003.jpg').
	self treeRadius: 1.0.! !

!QTreeBase methodsFor: 'initialize' stamp: 'bgf 9/21/2007 14:11'!
setSizeForTree: tree
	self treeRadius: ((tree boundingBox extentX max: tree boundingBox extentZ) / 2.2)! !

!QTreeBase methodsFor: 'initialize' stamp: 'bgf 10/12/2007 17:22'!
treeRadius: aScalar

	"Configure for a tree of the given overall canopy radius"
	self baseRadius: aScalar * 0.5;
		topRadius: aScalar * 0.04;
		height: aScalar*0.08;
		slices: 9;
		stacks: 3;
		uvScale: 2@0.25;
		translationX: self translation x y: (aScalar*-0.03) z: self translation z;
		solid: true;
		capped: true.! !

TGroup subclass: #QAnalogClock
	instanceVariableNames: 'hours minutes seconds case tzOffset radius'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QAnalogClock commentStamp: '<historical>' prior: 0!
An analog wall clock; the time is rendered at render-time by adjusting the rotation of the hands,
so the clock can happily render in local time for each different participant
without any space-disagreement complications.

The clock may have a tzOffset: minutes, 
in which case it will use that time zone
for all participants (relying on their local system time when rendering,
so there won't be exact agreement, just time-zone agreement.)

This object only provides the clock hands, not the face or ticks or casing.
A geometry object (the case) can be added to the clock via #case:,
and it will be persisted and form the body.!

!QAnalogClock class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QAnalogClock class methodsFor: 'properties' stamp: 'bgf 1/26/2010 11:57'!
addPropertyRows: propertyDialog forFrame: clockref

	propertyDialog addRow: (
		(QScalarPropertyRow  new)
			allowNil: true;
			titleString: 'TZ Offset';
			fetchBlock: [ :f | (f get: #tzOffset) ];
			acceptBlock: [ :v : f| f future tzOffset: v ] ;
			balloonHelp: 'Time Zone\Offset in minutes from GMT.\If blank, each user sees local time.' withCRs;
		yourself).

	propertyDialog addRow: (
		(QScalarPropertyRow new )
			titleString: 'Radius';
			fetchBlock: [ :f  | f get: #radius ] ;
			acceptBlock: [ :v :f| f future radius: v ] ;
			balloonHelp: 'Length of the second hand.';
		yourself).
	
	propertyDialog addRow: (
		(QTFramePropertyRow new)
			titleString: 'Clock Case';
			fetchBlock: [ :f | (f get: #case) ];
			acceptBlock: [ :v :f | f future caseFromProperties: v ] ;
			balloonHelp: 	
				('A geometry for the clock-case.\Setting this destroys the previous case.' withCRs);
			yourself).
	super addPropertyRows: propertyDialog forFrame: clockref.! !

!QAnalogClock class methodsFor: 'defaults' stamp: 'bgf 11/10/2009 16:19'!
defaultEventMask
	^ EventPointerOver + EventPointerDown! !

!QAnalogClock class methodsFor: 'defaults' stamp: 'bgf 6/4/2009 16:11'!
defaultIsFromTemplate
	"Most common use is decor in forums."
	^ true! !

!QAnalogClock class methodsFor: 'defaults' stamp: 'bgf 7/26/2008 15:23'!
defaultRadius
	^ 1.25! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 1/21/2009 20:16'!
case
	"The mesh defining the clock body, or nil."
	^ case! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 1/21/2009 20:38'!
case: aGeometry

	"A mesh which I will treat as a persistent child;
	 this should be everything about the clock except its hands.
	 Setting this destroys the previous case!!"

	(aGeometry = case) ifTrue: [ ^ self ].
	case ifNotNil: [
		case destroy.
	].
	aGeometry ifNotNil: [
		self addChild: aGeometry.
	].
	case := aGeometry.

	self shapeChanged.! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 1/21/2009 20:31'!
caseFromProperties: aGeometry

	"A mesh which I will treat as a persistent child;
	 this should be everything about the clock except its hands.
	 Setting this destroys the previous case!!

	 Adds validation and transform-reset to the basic accessor,
	 to support clumsy interactive case-setup using the property sheet.
	 (Drag a case model onto the property row from the inspector.) "

	(aGeometry = case) ifTrue: [ ^ self ].

	aGeometry ifNotNil: [
		 (aGeometry isOrContains: self) ifTrue: [ ^ self ].

		aGeometry localTransform: ((Matrix4x4 identity) 
			translation: (0@0@-0.01); 
			yourself).
	].
	self case: aGeometry.! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 11/4/2008 19:21'!
childrenForBrowser
	^ case ifNotNil: [ { case } ] ifNil: [ #() ].
! !

!QAnalogClock methodsFor: 'persist' stamp: 'bgf 7/26/2008 11:49'!
constructFrom: pStream

	super constructFrom: pStream.

	self case: (pStream readObject: 'case' default: case).
	radius := pStream readNumber: 'radius' default: radius.
	tzOffset := pStream readNumber: 'tzOffset' default: nil.! !

!QAnalogClock methodsFor: 'initialize' stamp: 'bgf 4/17/2009 11:45'!
defaultIsAtomic
	"So the clock case is listed in the browser."
	^ false! !

!QAnalogClock methodsFor: 'menu' stamp: 'bgf 10/25/2009 17:43'!
fillContextMenu: aMenu forEvent: anEvent
	"The clock menu allows choice between every-man-for-himself,
	 or else using a single time zone.  
 	 Note that the clock time itself is NOT replicated, just the zone,
	 so clocks will be skewed as user local times are skewed.
	 Hopefully, in the NTP era, this won't be a severe problem."

	(aMenu add: 'Show Local Times'
			target: self
			action: #tzOffset:
			arguments: { nil })
		checked: (tzOffset isNil).

	(aMenu add: 'Use My Time Zone'
			target: self
			action: #tzOffset:
			arguments: {anEvent avatar userInfo tzOffset})
		checked: (tzOffset = (anEvent avatar userInfo tzOffset)).		
! !

!QAnalogClock methodsFor: 'private' stamp: 'bgf 7/26/2008 16:13'!
handMaterial: bundledTexFile

	"Make the standard colorized xparent material for the given hand texture file."
	^ (TMaterial new)
		diffuseColor: Color white asVectorColor;
		ambientColor: (Color gray: 0.8) asVectorColor;
		texture: (QTextures find: bundledTexFile);
		transparency: 0.99;
		yourself.! !

!QAnalogClock methodsFor: 'initialize' stamp: 'bgf 11/10/2009 16:30'!
initialize
	super initialize.
	hours := QAnalogClockHand new.
	minutes := QAnalogClockHand new.
	seconds := QAnalogClockHand new.

	"The hand textures are assumed to be 
	 	- the same image size for all three hands
		   (so the hour hand is shorter within its texture)
		- pointing up in y, from a starting point just inside the lower edge,
		  so the longest hand (sweep second) fills its texture in y
		- overall 1x2 in proportion, so the extent is always (radius/2) @ radius
	  	  when radius is the face radius.
	The hand-rotation inside the QAnalogClockHand render method,
 	 and the layout in my updateShape method, rely on these assumptions."

	hours material: (self handMaterial: 'Content/Qwaq/Textures/ClockHourHand.png'); 
		   whichHand: #hour.
	minutes material: (self handMaterial: 'Content/Qwaq/Textures/ClockMinuteHand.png'); 
		   whichHand: #minute.
	seconds material: (self handMaterial: 'Content/Qwaq/Textures/ClockSecondHand.png'); 
		   whichHand: #second.

	self addChild: hours; addChild: minutes; addChild: seconds.

	self radius: self class defaultRadius.! !

!QAnalogClock methodsFor: 'menu' stamp: 'bgf 7/26/2008 21:11'!
menuTitle
	"Show what mode we're in... helps explain the choices."

	^ super menuTitle,
			(tzOffset 
				ifNil: [ ' - Local Time' ]
				ifNotNil: [  ' - ', (QStyles tzDescription: tzOffset) ]).! !

!QAnalogClock methodsFor: 'persist' stamp: 'bgf 7/26/2008 11:47'!
persistOn: pStream

	super persistOn: pStream.
	pStream storeObject: 'case' value: case default: nil.
	pStream storeNumber: 'radius' value: radius default: (self class defaultRadius).
	pStream storeNumber: 'tzOffset' value: tzOffset default: nil.! !

!QAnalogClock methodsFor: 'events' stamp: 'bgf 10/25/2009 12:49'!
pointerLeave: event
	"The floater lingers during mouse-over, so kill it fast on exit"
	event avatar ifNotNil: [
		event avatar clearFloatMessageForEvent: event.
	].
	! !

!QAnalogClock methodsFor: 'events' stamp: 'bgf 10/25/2009 12:49'!
pointerOver: event
 	self showFloatFor: event.
! !

!QAnalogClock methodsFor: 'scale' stamp: 'bgf 2/11/2010 20:21'!
privateScale: factor
	super privateScale: factor.
	radius := radius * factor.! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 7/26/2008 11:43'!
radius
	^ radius
! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 2/12/2010 01:15'!
radius: aScalar
	"Used to size the hands."
	radius := (aScalar ifNil: [1]) max: 0.1.	"Used elsewhere as divisor"
	self shapeChanged.
! !

!QAnalogClock methodsFor: 'hierarchy' stamp: 'bgf 11/4/2008 20:19'!
replaceChild: obj with: copy
	"Underpinning for convertFromThumb..."
	obj = case ifTrue: [
		self case: copy
	] ifFalse: [
		super replaceChild: obj with: copy
	].
! !

!QAnalogClock methodsFor: 'events' stamp: 'bgf 11/13/2008 13:04'!
showFloatFor: event

	"Show floating description message to the event sender"
	| dur str |

	event avatar ifNil: [ ^ self ].
	str := WriteStream on: String new.

	self timeToDepict asDateAndTime printHMSOn: str.
	str nextPut: $\.		"Float message linebreak"

	tzOffset ifNil: [
		str nextPutAll:  'Clock shows each person their local time.'

	] ifNotNil: [

		str nextPutAll:  'GMT'.

		dur := Duration minutes: tzOffset.
		dur negative ifTrue: [
			dur := dur negated.
			str nextPutAll: '-'.
		] ifFalse: [
			str nextPutAll: '+'.
		].
		dur hours < 10 ifTrue: [ str nextPut:$0 ].  dur hours printOn: str.
		str nextPut: $:.
		dur minutes < 10 ifTrue: [ str nextPut:$0 ].  dur minutes printOn: str.
		str nextPutAll: str.
	].

	event avatar showFloatMessage: str contents forEvent: event.! !

!QAnalogClock methodsFor: 'private' stamp: 'bgf 7/26/2008 16:12'!
timeToDepict
	
	"Return a Time or DateAndTime - 
	 this is side-effect free,
	 and can be used in render. "
	self tzOffset ifNotNilDo: [ :offset |
		^ (DateAndTime now utcOffset: (offset * 1000000000 * 60))
	] ifNil: [
		^ Time now.
	].
! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 7/26/2008 14:05'!
tzOffset
	"May be nil, to mean always render in local time."
	^ tzOffset! !

!QAnalogClock methodsFor: 'accessing' stamp: 'bgf 7/26/2008 14:34'!
tzOffset: minutes
	"May be nil, to mean always render in local time."
	tzOffset := minutes! !

!QAnalogClock methodsFor: 'hierarchy' stamp: 'bgf 7/26/2008 15:51'!
updateShape

	| depth |
	depth := self radius / 80.0.

	"The textures are specialized 1x2  shapes of the same texture size,
	 so that the minute hand fills the height of its own.
	 The QAnalogClockHand makes assumptions about the texture layout;
	 the starting point is that the textures are centered on the face."

	hours translation: (0@0@(4*depth)); extent: radius*0.5 @ radius.
	minutes translation: (0@0@(3*depth)); extent: radius*0.5 @ radius.
	seconds translation: (0@0@(3*depth)); extent: radius*0.5 @ radius.
	
! !

TGroup subclass: #QDoor
	instanceVariableNames: 'body knob angleWhenClosed angleWhenOpen currentTransition currentState knobName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QDoor commentStamp: '<historical>' prior: 0!
A simple clock-the-knob door mechanism.

Assumes it has a child mesh (also held as 'body') that is a doorlike object.
Further, there is a child within the body called #doorknob (or my #knobName)
and that opens and closes the door when you click it.

The door body should have its x-z origin at the hinge axis,
since I operate the door through a simple rotation of it.

The angle at closed, and the angle at open, are properties.
Typically these are 0 and 135, or 0 and -135 for the other-handed door.

Also, if I share a Group parent with exactly one other door,
then when I open or close, so does it.  (Auto double-door).


!

!QDoor class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QDoor class methodsFor: 'properties' stamp: 'bgf 1/26/2010 12:17'!
addPropertyRows: propertyDialog forFrame: doorRef

	propertyDialog addRow: (
		(QScalarPropertyRow new )
			titleString: 'Rotation When Closed';
			fetchBlock: [ : f | f  get: #angleWhenClosed ];
			acceptBlock: [ :v : f| f future angleWhenClosed: v ];
			balloonHelp: 'Degrees rotation around origin when closed.';
		yourself).
	propertyDialog addRow: (
		(QScalarPropertyRow new )
			titleString: 'Rotation When Open';
			fetchBlock: [  : f | f   get: #angleWhenOpen ];
			acceptBlock: [ :v  : f | f   future angleWhenOpen: v ];
			balloonHelp: 'Degrees rotation around origin when open.';
		yourself).
	propertyDialog addRow: (
		(QStringPropertyRow new )
			titleString: 'Name of Knob in Body';
			fetchBlock: [  : f | f   get: #knobName ];
			acceptBlock: [ :v  : f | f   future knobName: v ];
			balloonHelp: 'Object-name of the knob within my sub-tree.';
		yourself).

	super addPropertyRows: propertyDialog forFrame: doorRef.
! !

!QDoor class methodsFor: 'defaults' stamp: 'bgf 11/10/2009 15:56'!
defaultEventMask
	^ EventPointerOver + EventPointerDown! !

!QDoor methodsFor: 'hierarchy' stamp: 'bgf 7/27/2008 18:59'!
addChild: aFrame atIndex: i

	"Intercepted so reparenting in the inspector can make a door;
	 my first child is always my body unless you've hand-edited XML
	 or run code to change that."

	super addChild: aFrame atIndex: i.
	self body ifNil: [
		self body: aFrame
	].! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:08'!
angleWhenClosed
	"Degrees - the rotation around y of the body in the closed state."
	^ angleWhenClosed! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:07'!
angleWhenClosed: anAngle
	"Degrees - the rotation around y of the body in the closed state."
	angleWhenClosed  := anAngle! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:07'!
angleWhenOpen
	"Degrees - the rotation around y of the body in the open state."
	^ angleWhenOpen! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:07'!
angleWhenOpen: anAngle
	"Degrees - the rotation around y of the body in the open state."
	angleWhenOpen  := anAngle! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 14:39'!
body
	"The whole mesh tree describing the door."
	^ body! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 8/4/2008 00:35'!
body: aGeometry

	"A mesh (that must be my child);
	 I rotate this when the door is opened and closed."

	(aGeometry = body) ifTrue: [ ^ self ].
	body ifNotNil: [
		body := nil.
	].
	self knob: nil.

	aGeometry ifNotNil: [	
		body := aGeometry.
		self updateKnobFromName.
	].! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:00'!
close
	"Run the animation to close the door."

	| increment t |
	currentTransition := false.
	body ifNil: [ ^ self ].
	body rotationAroundY: (self angleWhenOpen).
	increment := (self angleWhenClosed - self angleWhenOpen)/10.0.
	1 to: 10 do: [ :i |
		(body future: (t := 50*i)) addRotationAroundY: increment.
	].
	(self future: t+1) setClosed.! !

!QDoor methodsFor: 'persist' stamp: 'bgf 7/27/2008 17:52'!
constructFrom: pStream

	| closed |
	super constructFrom: pStream.

	self knobName: (pStream readString: 'knobName' default: knobName).
	self children: (pStream readSequence:'children' default: self children).
	self body: (pStream readObject: 'body' default: body).

	self angleWhenOpen: (pStream readNumber: 'angleWhenOpen' default: angleWhenOpen).
	self angleWhenClosed: (pStream readNumber: 'angleWhenClosed' default: angleWhenClosed).

	((closed := (pStream readBool: 'isClosed' default: (self isClosed))) ~= self isClosed)
	 ifTrue: [
		closed ifTrue: [ self setClosed ] ifFalse: [ self setOpen ].
	]! !

!QDoor methodsFor: 'initialize' stamp: 'bgf 4/16/2009 18:36'!
defaultIsAtomic
	^ (self class ~~ QDoor)! !

!QDoor methodsFor: 'initialize' stamp: 'bgf 11/10/2009 16:32'!
initialize
	super initialize.

	angleWhenClosed := 0.
	angleWhenOpen := 135.
	currentState := false.		"False=closed, true=open"
	currentTransition := nil.	"Non-nil boolean when in transition."
	knobName := #doorknob.! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:01'!
isClosed
	"Do I think I'm closed?"
	^ currentState not! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:01'!
isClosing
	"True in the time interval while my close-animation is running."
	^ (currentTransition == false)! !

!QDoor methodsFor: 'testing' stamp: 'bgf 11/12/2009 10:54'!
isEditableGroup
	"Yes, so we can add the body via drag-drop."
	^ self isAtomic not! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:01'!
isOpen
	"Do I think I'm open?"
	^ currentState! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:01'!
isOpening
	"True during the interval my open-animation is running."
	^ (currentTransition == true)! !

!QDoor methodsFor: 'initialize' stamp: 'bgf 7/27/2008 19:11'!
knob: aFrame
	"Private (see updateKnobFromName).
	 Configure my doorknob for click behavior."

	self stopScript: #knobClicked:.
	knob := aFrame.
	knob ifNotNil: [
		self startScript: #knobClicked: when: { knob . #pointerDown }
	].! !

!QDoor methodsFor: 'events' stamp: 'bgf 7/27/2008 19:10'!
knobClicked: event
	"Mouse down on the knob.
	 Open or close the door.
	 Note - this is where double-door peering happens."

	currentTransition ifNotNil: [ ^ self ].

	self isOpen 
		ifTrue: [ self close. self peerDoor ifNotNilDo: [ : peer | peer close ]]
		ifFalse: [ self open. self peerDoor ifNotNilDo: [ : peer | peer open ]]! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:08'!
knobName
	"The objectName of the descendant of my body
	 that is the click-control to open and close the door."
	^ knobName! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:09'!
knobName: aString

	"The objectName of the descendant of my body
	 that is the click-control to open and close the door."

	knobName := aString.

	"And carry out the update - find the knob and set it up."
	self updateKnobFromName.! !

!QDoor methodsFor: 'state' stamp: 'bgf 7/27/2008 19:00'!
open

	"Run the animation to open the door."

	| increment t |

	body ifNil: [ ^ self ].
	currentTransition := true.
	body rotationAroundY: (self angleWhenClosed).
	increment := (self angleWhenOpen - self angleWhenClosed)/10.0.
	1 to: 10 do: [ :i |
		(body future: (t := 50*i)) addRotationAroundY: increment.
	].
	(self future: t+1) setOpen.! !

!QDoor methodsFor: 'hierarchy' stamp: 'bgf 7/27/2008 19:11'!
peerDoor
	"IFF I and one other door are in the same parent group,
	 opening one opens the other, and vice versa.
	 This returns my peer door if such can be found."

	| peer |
	(self parent notNil 
	 and:  [ self parent children size = 2 ])
	ifTrue: [
		peer := (self parent children first = self)	
			ifTrue: [ self parent children second ]
			ifFalse: [ self parent children first ].

		(peer class == self class) ifTrue: [ ^ peer ].
	].

	^ nil! !

!QDoor methodsFor: 'persist' stamp: 'bgf 7/27/2008 16:26'!
persistOn: pStream

	super persistOn: pStream.
	pStream storeSequence: 'children' value: self children default: #().
	pStream storeObject: 'body' value: body default: nil.
	pStream storeString: 'knobName' value: knobName default: #doorknob.
	pStream storeNumber: 'angleWhenOpen' value: angleWhenOpen default: (135).
	pStream storeNumber: 'angleWhenClosed' value: angleWhenClosed default: 0.
	pStream storeBool: 'isClosed' value: (self isClosed) default: true. 
! !

!QDoor methodsFor: 'events' stamp: 'bgf 10/25/2009 12:38'!
pointerEnter: event

 	self showFloatFor: event.
! !

!QDoor methodsFor: 'events' stamp: 'bgf 11/13/2008 13:08'!
pointerLeave: event

	knob ifNotNil: [ knob unhilite ].

	"The floater lingers during mouse-over, so kill it fast on exit"
	event avatar ifNotNil: [
		event avatar clearFloatMessageForEvent: event.
	]! !

!QDoor methodsFor: 'events' stamp: 'bgf 10/25/2009 12:38'!
pointerOver: event

 	self showFloatFor: event.
! !

!QDoor methodsFor: 'hierarchy' stamp: 'bgf 7/27/2008 18:58'!
removeChild: aFrame

	"Intercepted so if I lose my body, I reset that member."

	super removeChild: aFrame.
	aFrame = body ifTrue: [
		self body: ((self children)
					ifEmpty: [ nil ]
					ifNotEmpty: [ self children first ]).
	].! !

!QDoor methodsFor: 'state' stamp: 'bgf 4/19/2009 23:38'!
setClosed

	"Complete the close action, or initialize state to closed."
	body ifNotNil: [	
		body rotationAroundY: (self angleWhenClosed).
	].
	currentTransition := nil.
	currentState := false.

	self root signal: #doorClosed with: self.! !

!QDoor methodsFor: 'state' stamp: 'bgf 4/19/2009 23:38'!
setOpen

	"Complete the open action, or initialize state to open."
	body ifNotNil: [
		body rotationAroundY: (self angleWhenOpen).
	].
	currentTransition := nil.
	currentState := true.

	self root signal: #doorOpened with: self.! !

!QDoor methodsFor: 'events' stamp: 'bgf 11/13/2008 13:03'!
showFloatFor: event

	"Put up my floating mouse-over message."
	| msg verb |

	event avatar ifNil: [ ^ self ].
	knob ifNotNil: [
		verb := currentState caseOf: {
			[true] -> [ 'close' ].
			[false] -> [ 'open' ].
 		} otherwise: [
			'open or close' 
		].

		knob = body ifTrue: [
			msg := 'Click the door to ', verb.
		] ifFalse: [
			msg := 'Click the knob to ', verb, ' the door'.
			knob hilite.
		].

	] ifNil: [
		msg := 'This door is not yet operating.\The control knob has not been supplied.\See Properties....'
	].

	event avatar showFloatMessage: msg forEvent: event! !

!QDoor methodsFor: 'accessing' stamp: 'bgf 7/27/2008 19:13'!
updateKnobFromName

	"My knob-name or my body has changed...
	 If my body has a descendant with objectName = knobName,
	 then configure that as my knob.
	 Otherwise the clear the knob to nil."

	body ifNil: [ 
		^ self knob: nil
	].
	knobName ifNil: [ knobName := #doorknob ].
	self knob: ((body find: [ : f | f objectName = knobName ]) at: 1 ifAbsent:[nil]).

! !

TGroup subclass: #QFloorFan
	instanceVariableNames: 'blades stepping ribbons base rim head rY drY sound'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QFloorFan commentStamp: '<historical>' prior: 0!
A fan that sits on the floor.

Turn it on by clicking the area of the pushbuttons.
Has ribbons to show air flow.
Move by dragging the inner base.
Turn (if you're good) by grabbing the outer wire rim.
!

!QFloorFan class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QFloorFan class methodsFor: 'defaults' stamp: 'bgf 6/4/2009 17:27'!
defaultDoContextMenu
	"Fewer unintended menus this way."
	^ false! !

!QFloorFan class methodsFor: 'defaults' stamp: 'bgf 6/4/2009 16:12'!
defaultIsFromTemplate
	^ false! !

!QFloorFan methodsFor: 'persist' stamp: 'bgf 8/15/2007 17:48'!
constructFrom: pStream
	super constructFrom: pStream.
	(pStream readBool: 'stepping' default: false) ifTrue: [
		self future startFan	"Future, to allow the initial ribbon updateShape to deploy masses"
	].
				
! !

!QFloorFan methodsFor: 'accessing' stamp: 'bgf 8/16/2009 00:46'!
dragAdapter
	^ base! !

!QFloorFan methodsFor: 'initialize' stamp: 'bgf 3/16/2010 15:54'!
initialize
	| model  shadow |

	super initialize.

	stepping := false.

	"Used to track stepping rotation of entire oscillating head assembly..."
	rY := 0.
	drY := 1.

	"This loads the frame-thumb; the real fan is a resource."
	model := TSceneReader new loadFileNamed: 'Objects/Qwaq/FloorFan.c3x'.
	self addChild: model.

	"The parts in the thumb are named to match the real meshes..."
	head := (model find: [:cf | cf objectName = 'HeadAssembly']) first.
	blades := (model find: [:cf|cf objectName = 'Blades']) first.
	blades doesClickSelect: false.

	base := QMotionAdapter 
				forLuggage: self 
				grip: (model find:[:cf| cf objectName = 'Base']) first.
	base isGroundBound:true.

	rim := QPlanePivot 
				forLuggage: self 
				grip: (model find: [:cf | cf objectName = 'Cowling']) first.

	"The area where we click to turn on/off"
	(model find: [:cf| cf objectName = 'Controls']) do: [:frame|
		self runScript: #pointerClick: when: {frame . #pointerDown}.
		frame doesClickSelect: false.
	].

	"And stick on some pretty ribbons."
	ribbons := OrderedCollection new.
	-1 to: 1 by: 2 do: [ :i|
		| ribbon |
		ribbon := QRibbon new.
		ribbon windOn:false.			"The ribbon is alive, but no wind means they dangle."
		ribbon extent:2@10.
		ribbon linkLength:0.22; linkWidth:0.12.
		ribbon translation: blades translation + (0.06+(i*0.6)@0@(0.1)).
		ribbon rotationAroundY: 180.
		ribbon stepTime: (60 + (10*i)).
		head addChild: ribbon.
		ribbons add: ribbon.
		ribbon doesClickSelect: false.		"So it is not in the visible-bounding-box."
	].

	shadow := QMaterials newRadialShadow.
	shadow addRotationAroundX:-90.
	shadow translationX: 0 y:0.02 z: 0.
	shadow extent: 1.2@1.2.
	self addChild: shadow.

	self addChild: (
		sound := (QSoundFrame new)
			soundEffectName: 'fan.wav';
			gain: 0.15;
			referenceDistance: 4;
			maxDistance: 300.0;
			visible: false;
			yourself).! !

!QFloorFan methodsFor: 'initialize' stamp: 'bgf 10/22/2008 00:22'!
isGroundBound
	"So the default creation paths put fans on the ground.  'Floor'Fan, after all."
	^ true! !

!QFloorFan methodsFor: 'accessing' stamp: 'bgf 6/2/2009 09:34'!
isPlacementLocked: bool
	super isPlacementLocked: bool.
	base isEnabled: (bool not).! !

!QFloorFan methodsFor: 'persist' stamp: 'bgf 2/23/2007 12:59'!
persistOn: pStream
	super persistOn: pStream.
	pStream storeBool: 'stepping' value: stepping default: false.! !

!QFloorFan methodsFor: 'events' stamp: 'bgf 8/15/2007 17:28'!
pointerClick: event

	stepping ifTrue: [self stop] ifFalse: [self startFan].
	QSoundEffects play:#click for:self.! !

!QFloorFan methodsFor: 'stepping' stamp: 'bgf 2/22/2007 17:56'!
start

	"We don't start just by being added to the space;
	 we have a specific control for starting."! !

!QFloorFan methodsFor: 'stepping' stamp: 'bgf 8/15/2007 17:38'!
startFan

	stepping := true.
	(self future:50) step.
	ribbons do: [:ribbon | ribbon stop; windOn: true; restart; start].
	sound play; isLooping: true.
! !

!QFloorFan methodsFor: 'stepping' stamp: 'bgf 2/22/2007 22:00'!
step

	stepping ifTrue: [

		(rY + drY) abs > 45 ifTrue: [
			drY := drY negated.
		].
		rY := drY + rY.

		blades addRotationAroundZ:-29.
		head rotationAroundY: rY.
		(self future:65) step.
	].! !

!QFloorFan methodsFor: 'stepping' stamp: 'bgf 2/23/2007 19:32'!
stop
	"Stop the fan, and turn off the wind to the ribbons."

	stepping ifTrue: [
		stepping := false.
		ribbons do: [:ribbon | ribbon windOn: false].

		"Looks really weird if the blades just halt, so do a lame spin-down."
		(blades future: 80) addRotationAroundZ:-25.
		(blades future: 160) addRotationAroundZ:-18.
		(blades future: 240) addRotationAroundZ:-10.
		(blades future: 320) addRotationAroundZ:-5.
	].

	sound stop.! !

TGroup subclass: #QForumCeiling
	instanceVariableNames: 'vault1 vault2 vault3 vault4 upper lower extent lowerMaterial lowerUV vaultColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QForumCeiling commentStamp: '<historical>' prior: 0!
A simple articulated ceiling for forum-sized rooms.

Consists of a 'vault' - just four angled rectangles at the edges,
an upper rectangle - filling in the area above the vault,
and a lower rectangle - suspended mid-height, forming the interior breadth.

The extent is always given as an xyz vector; 
	y is the lift from wall edge to upper extent;
	x and z are from the outer bases of the ceiling structure.

Translation should place the origin of the ceiling at the
center of the plane crossing the tops of the room's walls.

Configurable persistent properties for appearance: 

	lowerMaterial and uvScale control the main sheet of the ceiling.	
	vaultColor controls the color of the sloping vaults.
!

!QForumCeiling class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumCeiling class methodsFor: 'inspector' stamp: 'sn 3/8/2013 17:08'!
addPropertyRows: propertyDialog forFrame: frameref
	"The base version of the QPropertyDialog row-population protocol;
	 add property rows for the user-editable aspects of this frame."

	self addNameRowIfNeeded: propertyDialog forFrame: frameref.

 	propertyDialog addRow: (
		(QVector3PropertyRow new)
			titleString: 'Extent'translateMe;
			fetchBlock: [:f | f get: #extent ];
			acceptBlock: [:ext :f| f future extent: ext ];
			balloonHelp: 'Dimensions of the ceiling.'translateMe;
		yourself).

	super addPropertyRows: propertyDialog forFrame: frameref.! !

!QForumCeiling class methodsFor: 'defaults' stamp: 'bgf 9/21/2008 16:03'!
defaultDoContextMenu
	"Fewer unintended menus this way.  (Means you need the inspector for editing.)"
	^ false! !

!QForumCeiling class methodsFor: 'defaults' stamp: 'bgf 6/1/2010 12:11'!
defaultExtent
	^ 10@1@10! !

!QForumCeiling class methodsFor: 'defaults' stamp: 'bgf 6/4/2009 16:12'!
defaultIsFromTemplate
	"Reduces xml volume since ceilings are most always terrain."
	^ true! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 7/14/2007 17:30'!
buildFrames

	"This is done lazily, upon first 'updateShape'
	 to minimize activity during construction."

	vault1 := TRectangle new.
	vault1 material: self vaultMaterial;
			addRotationAroundX: 45.0.
	self addChild: vault1.
	vault2 := TRectangle new.
	vault2 material: self vaultMaterial;
		    uvAngle: 180.0;
			addRotationAroundX: 135.0.
	self addChild: vault2.

	vault3 := TRectangle new.
	vault3 material: self vaultMaterial;
			addRotationAroundY: 90;
			addRotationAroundX: 45.0.
	self addChild: vault3.
	vault4 := TRectangle new.
	vault4 material: self vaultMaterial;
			addRotationAroundY: 90;
			addRotationAroundX: 135.0;
			uvAngle: 180.
	self addChild: vault4.

	upper := TRectangle new.
	upper divisions: 2@2;
		   rotationAroundX: 90;
		   material: self upperMaterial.
	self addChild: upper.

	lower := TRectangle new.
	lower divisions: 6@6;
		   rotationAroundX: 90;
		   material: self lowerMaterial clone.
	self addChild: lower.! !

!QForumCeiling methodsFor: 'persist' stamp: 'bgf 6/1/2010 12:12'!
constructFrom: pStream

	super constructFrom:pStream.
	self extent: (pStream readVector3: 'extent' default: (self class defaultExtent)).
	self lowerMaterial: (pStream readObject: 'lowerMaterial' default: self defaultLowerMaterial).
	self lowerUVScale: (pStream readPoint: 'lowerUV' default: nil).
	self vaultColor: (pStream readColor: 'vaultColor' default: nil).! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 7/14/2007 13:55'!
defaultLowerMaterial

	^  (TMaterial new)
		color: (Color r:0.9 g:0.9 b:0.9);
		emissiveColor: (Color r:0.1 g:0.1 b: 0.1);
		texture: (QTextures find: 'ceilingBroad.jpg');	
		yourself.
! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 11/27/2009 20:35'!
defaultVaultColor

	^ QColors warmWhiteColor! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 03:35'!
extent
	^ extent! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 14:58'!
extent: whd
	"The extent  x and z are width and breadth; y is the vault depth.
	 (The Ceiling should be translated so that it's 0,0,0 is the upper center of the room.)"
	
	extent := whd.
	self shapeChanged.! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 6/1/2010 12:11'!
initialize

	super initialize.
	lowerUV := nil.
	extent := self class defaultExtent.
	self shapeChanged.! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 16:23'!
lowerMaterial

	"The material of the (extent - gap) suspended rectangle - 
	 this is most of the visible ceiling."

	^ lowerMaterial ifNil: [ self defaultLowerMaterial ]! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 14:11'!
lowerMaterial: aMaterial

	lowerMaterial := aMaterial clone.
	lowerMaterial emissiveColor: (Color r:0.12 g:0.12 b:0.12).
	lower ifNotNil: [
		lower material: self lowerMaterial clone.
	].
! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 12:44'!
lowerUVScale
	"The uvscale for the cube representing the bulk of the ceiling."
	^ lowerUV ifNil: [ (self extent x @ self extent z ) / 6.0 ].! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 16:25'!
lowerUVScale: uvs	
	"The cube representing the bulk of the ceiling."
	(uvs = self lowerUVScale) ifFalse: [
		lowerUV := uvs.
		lower ifNotNil: [
			lower uvScale: self lowerUVScale
		].
	].! !

!QForumCeiling methodsFor: 'persist' stamp: 'bgf 6/1/2010 12:11'!
persistOn: pStream
	| |
	super persistOn:pStream.
	pStream storeVector3: 'extent' value: extent default: (self class defaultExtent).
	pStream storeObject: 'lowerMaterial' value: self lowerMaterial default: self defaultLowerMaterial.
	pStream storePoint: 'lowerUV' value: lowerUV default: nil.
	pStream storeColor: 'vaultColor' value: vaultColor default: nil.
! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 2/11/2010 15:56'!
privateScale: aFactor
	"We intercept scale and recompute the ceiling structure."

	nomScale := nomScale * aFactor.
	self extent: (self extent * aFactor).
	self shapeChanged.! !

!QForumCeiling methodsFor: 'hierarchy' stamp: 'bgf 7/14/2007 13:21'!
updateShape

	| inset x z hyp |

	upper ifNil: [ self buildFrames ].

	x := (extent x / 2.0).
	z := (extent z / 2.0).

	inset := (extent y).
	hyp := inset * 1.414214.		"1 / cos 45"

	upper extent: (extent x @ extent z) - (2*inset);
		   translation: 0@(extent y)@0.

	lower extent: (extent x - (3*inset)) @ (extent z - (3*inset));
		   translation: (0@(extent y / 2) @ 0).

	vault1 extent: extent x @ hyp;
		    translation: 0@(inset/2.0)@ (z negated +(inset/2.0)).

	vault2 extent: extent x @ hyp;
		    translation: 0@(inset/2.0)@ (z-(inset/2.0)).

	vault3 extent: extent z @ hyp;
		    translation: (x negated +(inset/2.0))@(inset/2.0)@0 .

	vault4 extent: extent z @ hyp;
		    translation:  (x-(inset/2.0))@(inset/2.0)@0.

	upper uvScale: upper extent / 2.0.
	lower uvScale: self lowerUVScale.! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 11/27/2009 20:36'!
upperMaterial

	^  (TMaterial new)
		color: self vaultColor;
		emissiveColor: (QColors gray5);
		yourself.
! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 16:23'!
vaultColor
	^ vaultColor ifNil: [ self defaultVaultColor ]! !

!QForumCeiling methodsFor: 'accessing' stamp: 'bgf 7/14/2007 17:44'!
vaultColor: vC
	vaultColor := vC.
	vault1 ifNotNil: [
		{vault1 . vault2 . vault2. vault4 } do: [ :v |
			v material color: self vaultColor.
		].
		upper material color: self vaultColor.
	].! !

!QForumCeiling methodsFor: 'initialize' stamp: 'bgf 7/14/2007 17:02'!
vaultMaterial

	^  (TMaterial new)
		color: self vaultColor;
		emissiveColor: (Color r:0.15 g:0.15 b:0.15);
		texture: (QTextures find: 'vault-plain.jpg');	
		yourself.
! !

TGroup subclass: #QPeeringDesk
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QPeeringDesk commentStamp: '<historical>' prior: 0!
A leftover, held in the system so ancient forums that refer to it in persistence can be read.!

!QPeeringDesk class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

TSimpleRectangle subclass: #QAnalogClockHand
	instanceVariableNames: 'hand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QAnalogClockHand commentStamp: '<historical>' prior: 0!
A specialized rectangle, private child of QAnalogClock.
Gives itself a private render-time rotation
to show a clock hand's current position based on the parent's time.!

!QAnalogClockHand class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/26/2008 14:20'!
currentAngle

	"Return the angle I should be rendered at (degrees; 0 means high-noon vertical), 
	 based on timeToDepict, and on which hand I am."

	^ hand caseOf: {
		[#minute] -> [ self currentMinuteAngle ].
		[#second] -> [ self currentSecondAngle ].
	} otherwise: [
		self currentHourAngle
	].! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/26/2008 14:19'!
currentHourAngle

	"Return the uvAngle I should be rendered at, for my parent's current tzOffset."

	| mins t |
	t := self timeToDepict.
	mins := ((t hours \\ 12)*60) + t minutes.
	^  mins / 2.0! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/26/2008 14:19'!
currentMinuteAngle

	"Return the angle I should be rendered at, for my parent's current tzOffset."

	^  (self timeToDepict) minutes * 6.0! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/26/2008 14:19'!
currentSecondAngle

	"Return the uvAngle I should be rendered at, for my parent's current tzOffset."

	^ Time now seconds * 6! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/27/2008 10:34'!
renderPrimitive: ogl alpha: bAlpha

	"I render myself with a twist - literally...
	 rendering applies the hand-angle rotation as part of rendering,
	 so the time-based hand rotation is off-island.
	 (There is no replicated state involved in portraying user-local time.)"

	| rm cos sin hyp a rad |

	"The hand texture is assumed to be 
	 	- the same image size for all three hands
		   (so the hour hand is shorter within its texture)
		- pointing up in y, from a starting point just inside the lower edge,
		  so the longest hand (sweep second) fills its texture in y."

	a := self currentAngle.
	rad := a degreesToRadians.
	sin := rad sin.
	cos := rad cos.
	hyp := self extent y * 0.48.		"I.e. slide into centerpoint by 2%"

	rm := Matrix4x4 identity.
	rm translationX: (hyp * sin) y: (hyp * cos) z: 0.

	"Inline rotationAroundZ: -- since we've already done the trig."
	rm a11: cos .
	rm a12: sin .
	rm a22: cos .
	rm a21: sin negated.

	"Given the way render-alpha currently works, the push-pop is superflous,
	 since the current matrix is popped right after the doRenderAlpha stuff,
	 but that might change.  
	 Having these changes be more than local would create strange mysteries,
	 so we take the perf hit and do it safely."
	ogl glPushMatrix.
	ogl glMultTransposeMatrixf: rm.
		super renderPrimitive: ogl alpha: bAlpha.
	ogl glPopMatrix.
! !

!QAnalogClockHand methodsFor: 'render' stamp: 'bgf 7/26/2008 14:18'!
timeToDepict
	
	"Return a Time or DateAndTime;
	 if I'm in a clock, I ask my parent; if not, I just use local time."

	^ (self parent class includesBehavior: QAnalogClock)
		ifTrue: [ self parent timeToDepict ]
		ifFalse: [ Time now ]
! !

!QAnalogClockHand methodsFor: 'accessing' stamp: 'bgf 7/26/2008 14:22'!
whichHand: hms
	" #hour, #minute, or #second "
	hand := hms.! !

QSpace subclass: #QConferenceRoom
	instanceVariableNames: 'extent lights compassRotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums'!

!QConferenceRoom commentStamp: '<historical>' prior: 0!
This space is the basic enclosed collaboration workspace - a forum room.
Usually created and populated by a QForumTemplate.

Four lights are used; colors are light warm, neutral.
Provides mostly even light with a bit of 3-D modelling arising from the hue variations.
(Need even light since the display panels and app panes face in all directions.)!

!QConferenceRoom class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QConferenceRoom class methodsFor: 'defaults' stamp: 'bgf 1/31/2007 23:54'!
defaultExtent

	"Don't change this - it controls the avatar box of legacy forums;
	 change the extent assignment in 'initialize' to a non-default instead.
	 That will mean later forums always persist their extent,
	 but gives backwards compatibility."

	^ 60@11@60! !

!QConferenceRoom class methodsFor: 'defaults' stamp: 'ar 4/16/2011 17:10'!
genericName
	"For better labeling of property-sheet parent rows
	 (Conference room is too specific for some forums like library)."
	^ 'Forum'! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 4/3/2009 09:48'!
actionVerbs
	"Selectors to use in the action menu; available to all visitors."

	^ super actionVerbs,
	{
		{#toggleMainLights . (self lightsAreOn ifTrue: [ 'Lights Off'] ifFalse: [ 'Lights On' ])} .
		{#resetLights . 'Reset Lighting'}
	},
	(self ceiling ifNotNil: [
		{
			{ #separator }. 
			{#raiseCeiling . 'Increase Room Height' }
		},	
		( (self extent y > 6) 
			ifTrue: [  {{#lowerCeiling . 'Decrease Room Height' }}] ifFalse: [ #() ] )
	 ] ifNil: [ #() ] ).! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 2/15/2007 13:38'!
addCard: description toRolodex: rolodex

	^ rolodex insertCard: (self makeContent: description).
	! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 2/6/2008 18:32'!
adjustHeightBy: yUnit

	| ceiling walls |
	ceiling := self ceiling.
	ceiling ifNotNil: [
		ceiling translation: (ceiling translation + (0@yUnit@0)).
	].

	walls :=  self children select: [ : f | f isKindOf: QWall ].
	walls do: [ :wall | 
		(wall extent y - extent y) abs < 1 ifTrue: [
			wall extent: (wall extent + (0@yUnit)).
		].
	].

	extent := extent + (0@yUnit@0).! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 2/6/2008 18:20'!
ceiling
	"First QForumCeiling in this space, if any."
	^ self children detect: [ : fc | fc isKindOf: QForumCeiling ] ifNone: [ nil ].! !

!QConferenceRoom methodsFor: 'persist' stamp: 'bgf 6/2/2009 12:50'!
checkForRepairCase: child
	"Extension:
 	 Some macro-repair cases specifically for QConferenceRoom instances.
       The first case that returns true will end the checks...."
	true caseOf: {
		[ self repairProjectForumRug: child ] -> [] .
		[ self repairRails: child ] -> [] .
		[ self repairReceptionFlags: child ] -> [] .
	} otherwise: [
		super checkForRepairCase: child. 
	].
! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/3/2009 15:36'!
checkLibraryTemplateFlags
	"Check and upgrade a Library (V2) instance
	 that predates the isFromTemplate: flag.    
	 Uses a ceiling mesh as marker."

	self islandName = 'Library' ifFalse: [ ^ false ].
	(self children anySatisfy: [ : f |
 		f class = TFrameThumb 
		and: [ f objectName = 'ArcBeamedCeiling' or: [ f objectName = 'BeamedCeiling'] ]
		and: [ f isFromTemplate not ] ])
	ifTrue: [
		self applyTemplateFlagAggressively.

		"Cleanup since the some library template generations omitted this: "
		self children select: [ : f | f class = QDisplaySign ] 
					thenDo: [ : f | f isPlacementLocked: true ].
		QLogger logInfo: 'Updated library for template flagging.'.
		^ true
	].

	^ false! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/3/2009 11:10'!
checkProjectTemplateFlags
	"Check and upgrade a Project (Baltic/Pacific/Caribbean) instance
	 that predates the isFromTemplate: flag.    
	 Uses the pushpin mesh (decoration on the brainstorming wall) as marker."

	self islandName = 'Forum' ifFalse: [ ^ false ].
	(self children anySatisfy: [ : f |
 		f class = TFrameThumb
		and: [ f isFromTemplate not ]
		and: [ 
			f sha = 'QYK9K3ZTAUX2QD1JPGSSHP728BM5YLR'  "The pushpin thingies" 
			or: [ f sha = 'PNMZTJNXKE0LX0HK5JOUWXZ1UU3JQRI' ] ]])
	ifTrue: [
		self applyTemplateFlagAggressively.
		self children 
			select: [ : c | c class = QBrainstormingBoard ] 
			thenDo: [ : c | c isPlacementLocked: true ].
		QLogger logInfo: 'Updated project-forum for template flagging.'.
		^ true
	].


	^ false! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/3/2009 11:10'!
checkReceptionTemplateFlags
	"Check and upgrade a Reception instance
	 that predates the isFromTemplate: flag.    
	 Uses the floor rug as marker."

	self islandName = 'Reception' ifFalse: [ ^ false ].
	(self children anySatisfy: [ : f |
 		f class = TGeometry
		and: [ f objectName = 'Reception-Rug' ]
		and: [ f isFromTemplate not ] ])
	ifTrue: [
		self applyTemplateFlagAggressively.
		QLogger logInfo: 'Updated Reception for template flagging.'.
		^ true
	].

	^ false! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 10/29/2006 07:28'!
compassRotation
	"A hint to navigation displays: the rotation from this room's negative-z to user North"
	^ compassRotation! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 10/29/2006 07:28'!
compassRotation: cr

	"A hint to navigation displays: the rotation from this space's negative-z to user North."
	compassRotation := cr! !

!QConferenceRoom methodsFor: 'persist' stamp: 'bgf 9/8/2007 14:41'!
constructFrom: pStream
	| |
	super constructFrom:pStream.
	self compassRotation:  
		(pStream readNumber: 'compassRotation' default: compassRotation).
	self extent: (pStream readVector3: 'extent' default: (self class defaultExtent)).

	"The avatar box is separate from extent so that custom hand-edited templates
	 can position it off-center; the normal extent-computed box is centered on the origin."
	avatarBox := pStream readObject: 'avatarBox' default: (self normalAvatarBox).

	"Recover the lights, which are statically created, so the lights-on action works."
	lights addAll: (self find: [ :fc | fc isKindOf: TLight]).

	"Patch this misfortune in older forums."
	self fogDensity = 0.003 ifTrue: [ 
		self fogDensity: 0.001.
	].
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 17:05'!
displays
	^ self frameChildren select: [:c|c isKindOf:QDisplayPanel]! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 22:52'!
entryGate
	"the main entry gate into this room"
	^ self gates first! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 22:52'!
entryLink: postcard
	self entryGate postcardLink: postcard! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 22:53'!
entryPortal
	^ self entryGate contents! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 23:00'!
exitGate
	^ self gates at: 2 ifAbsent: [nil]! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 23:01'!
exitLink: postcard

	"Link the exit gate to the given postcard."
	self exitGate postcardLink: postcard! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 23:01'!
exitPortal
	"The TPortal in the exit gate."
	^ self exitGate contents! !

!QConferenceRoom methodsFor: 'initialize' stamp: 'bgf 9/6/2007 10:46'!
extent
	"This is the room extent - the extent box defined by the template."
	^ extent! !

!QConferenceRoom methodsFor: 'initialize' stamp: 'bgf 3/11/2007 15:29'!
extent: aVector3
	"It is only useful to set this -before- populate, in early world-building;
	 it controls the creation of the walls, floor, etc."

	extent := aVector3.
	self fogDensity: 0.003 * ((100/extent max) min: 1).! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 9/1/2007 17:39'!
floor
	"My QFloor, if any."
	^ self childNamed: 'floor'! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 22:52'!
gates
	"All the QGates in this room"
	^ self find: [:c| c isKindOf: QGate]! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'das 3/12/2007 15:25'!
initialAvatarTranslation: trans

	^ trans x @ 5.5 @ trans z



	! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'das 3/12/2007 15:30'!
initialAvatarYaw: y

	^ y.! !

!QConferenceRoom methodsFor: 'initialize' stamp: 'bgf 10/19/2008 21:25'!
initialize

	| |
	super initialize.

	"For backwards compatibility; old QConferenceRoom hardcoded this result."
	self homeTransform: (Matrix4x4 new translation: -1@5.5@5).
	lights := OrderedCollection new.

	"First space in this island?"
	(Processor activeIsland at: #mainEntry) ifNil: [
		"Register the default island entry point."
		self registerDefaultMainEntry.
	].

	compassRotation := 0.
	self color: #(0.35 0.55 0.5) asVectorColor.
	self fogColor: #(0.48 0.55 0.63) asVectorColor.
	self fogDensity: 0.001.
	self fogOn: true.
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 4/3/2009 09:52'!
lightsAreOn
	"Are the standard lights on?  See toggleMainLights"
	lights size > 0 ifTrue: [
		^ self hasChild: (lights first) 
	].
	^ true	"Don't care... the actions won't do anything"! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/15/2007 10:07'!
linkEntryPortal: postcard label: aString

	| |
	self entryGate postcardLink: postcard.
	aString ifNotNil: [
		self entryGate label: aString.
	] ifNil: [
		self entryGate label: 'Portal'.
	].! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 23:02'!
linkExitPortal: postcard label: aString

	| |
	self exitGate future postcardLink: postcard.
	aString ifNotNil: [
		self exitGate label: aString.
	] ifNil: [
		self exitGate label: 'Portal'.
	].! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 1/24/2008 17:31'!
lowerCeiling

	self adjustHeightBy: ((self extent y/6.0) min: 10.0 max: 1.0) negated.
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 10/19/2008 14:02'!
makeContent: description

	"Generate card/shade content as described;  the result is QFileCard.
	 This supports easy hardcoded population of demo spaces
	 (allowing {...} arrays to describe all the content of a room.)
	 It is not intended to be a supported public API;
	 use the QFileCard instance creation methods instead."

	description first = #url ifTrue: [
		^ QFileCard forURL: (description second) 
	].

	description first = #mpegFile ifTrue: [
		^ QFileCard forMPEGFile: (description second) sha: (description third).
	].
	
 	description first = #text ifTrue: [
		^ QFileCard forText: description second
	].

	description first = #texture ifTrue: [
		^ QFileCard forTexture: (QTextures find: description second)
	].

	description first = #portal ifTrue: [
		^ QFileCard forPostcard: (TPostcard urlString:description second)
	].

	self error: 'Unknown content tag: ', description first.
	^nil.! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 5/14/2007 23:29'!
normalAvatarBox
	"the extra factor accounts for wall thickness and keeps us visually within...
	 the limit on how far inward comes from the need to keep portal gates close to the wall."
	| inset |
	inset := 0.6.
	^ TBox min: ((extent x/2-inset)@-1@(extent z/2-inset))negated
			max: ((extent x/2-inset)@(extent y)@(extent z/2-inset)).
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 10/4/2007 12:19'!
panelTranslationFromSpec: spec

	| offset xadj zadj rotation |

	offset := 0.25.	"The original positions are given in mid-wall basic coordinates;
				        we want a place for the origin of the panel object;
 				        so the offset sums the wall thickness + panel thickness, approximately."

	"A spec is (x@y@z) . yaw Degrees."

	rotation := spec second.
	xadj := (rotation degreeSin * offset).
	zadj := (rotation degreeCos * offset).
	^  spec first + (xadj@0@zadj).
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 15:39'!
pedestals
	^ self find: [:c | c isKindOf: QPedestal]! !

!QConferenceRoom methodsFor: 'persist' stamp: 'bgf 5/14/2007 23:33'!
persistOn: pStream
	| check |
	super persistOn:pStream.
	pStream storeNumber: 'compassRotation' value: compassRotation default: 0.
	pStream storeVector3: 'extent' value: extent default: (self class defaultExtent).

	"There is no equality testing on avatar boxes, so we do our own."
	check := self normalAvatarBox.
	(avatarBox min = check min and: [avatarBox max = check max]) ifFalse: [
		pStream storeObject: 'avatarBox' value: avatarBox
	].
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/2/2009 09:30'!
placeCeiling: layout
	| ceiling depth |
	layout ceilingMaterial ifNil: [
		^ self
	].
	depth := ((self roomWidth min: self roomDepth)/16.0) min: (self roomHeight/2.5).
	ceiling := (QForumCeiling new)
				extent: (self roomWidth) @  depth @ (self roomDepth);
				translation: 0 @ self roomHeight @ 0;
				lowerMaterial: layout ceilingMaterial;
				lowerUVScale: layout ceilingUVScale;
				yourself.
	layout vaultColor ifNotNil: [
		ceiling vaultColor: layout vaultColor.
	].
	ceiling isPlacementLocked: true.
	self addChild: ceiling.! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 10/30/2008 17:41'!
placeDisplayPanel: spec

	| panel trans |

	trans := self panelTranslationFromSpec: spec.

	"A spec is (x@y@z) . yaw Degrees."

	panel := QDisplayPanel withSimpleFrame.
	panel extent: 7.6@5.7.
	panel translation: trans.
	panel yaw: spec second.
	self addChild: panel.

	^ panel! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 5/31/2009 19:52'!
placeDisplayPanels: specs

	specs do: [ :spec | 
		| panel |
		panel := self placeDisplayPanel: spec.
		panel isPlacementLocked: true.
		panel isTemplateExclusion: true.
	].	
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/1/2009 21:48'!
placeFans: specs
	"Floor-standing fans."
	| fan |
	specs do: [:spec|
		fan := QFloorFan new.
		fan translation: (spec first x)@0@(spec first y).
		fan rotationAroundY: spec second.
		self addChild: fan.
		fan isTemplateExclusion: true.
	].! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/1/2007 18:45'!
placeFloor: layout

	| floor |
	floor := (QFloor new)
		translationX:0 y:(QFloor defaultThickness/-2) z: 0;
		extent: (extent x +2)@(extent z + 2) ;
		material: (layout floorMaterial);
		uvScale: layout floorUVScale;
		objectName: 'floor' copy;
		yourself.
	self addChild: floor.
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 5/31/2009 05:13'!
placeFloorPaths: specs material: mat layout: layout

	| strip |

	"Each spec entry is center . extent . rotation"
	mat ifNil: [ ^ self ].
	specs  do: [ :spec |
		strip := self floor addRugAt: spec first 
						extent: spec second asPoint
						material: mat.
		strip addRotationAroundZ: (spec third).	"really y, but x is rotated already."
		layout ifNotNil: [
			strip uvScale: (layout floorPathUVScale: spec).
		].
		strip objectName: ('runner' copy).
		strip doContextMenu: false.
	].	
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/2/2009 09:28'!
placeGate: spec
	| gate xadj zadj rotation |

	"Each spec entry is (x@y) . rotationDegrees . [ globalSymbol ] "

	rotation := spec second.
	xadj := (rotation degreeSin * QGate wallStandoff).
	zadj := (rotation degreeCos * QGate wallStandoff).

	gate := QGate new.
	gate translation: (spec first x@0@spec first y) + (xadj@0@zadj).
	gate rotationAroundY: spec second.
	self addChild: gate.
	gate closeContents.
	(spec size > 2) ifTrue: [
		gate registerGlobal: spec third.
	].

	^ gate! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/6/2007 14:52'!
placeGates: specs

	"Place the portal gates defined in the given list of specs;
	 see placeGate for the spec elements."
	specs do: [ :spec | 
		self placeGate: spec.
	].	
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/6/2007 15:25'!
placeLandmarks: specs

	"The layout's #landmarks, each a preset place marker in the space."
	specs  do: [ :spec |
		self addLandmark: spec first at: spec second facing: spec third sort: spec fourth
	].! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 1/2/2009 12:48'!
placeLights

	|  width light |

	width := self roomWidth + 10.

	"cool light in northwest corner."

	light := TLight new.
	light type: #directional;
		objectName: #northwestLight;
		 ambientColor: #(0.29 0.32 0.35 1);
		 diffuseColor: #(0.36 0.39 0.42 1);
		 specularColor: #(0.21 0.24 0.27 1);
		translationX: width*-1 y:2*width z: (width*-1);
		pitchYawRoll: 120@45@0;
		solid: false.
	lights add: light.
	self addChild: light.

	"warm light  opposite."

	light := TLight new.
	light type: #directional;
		objectName: #southeastLight;
		 ambientColor: #(0.35 0.32 0.29 1);
		 diffuseColor: #(0.42 0.39 0.36 1);
		 specularColor: #(0.22 0.19 0.16 1);	
		translationX: width y:2*width z: 1.5;
		pitchYawRoll: 120@-90@0;
		solid: false.
	lights add: light.
	self addChild: light.

	" fainter green light in third corner."

	light := TLight new.
	light type: #directional;
		 objectName: #southwestFill;
		 ambientColor: #(0.15 0.15 0.15 1);
		 diffuseColor: #(0.07 0.07 0.07 1);
		 specularColor: #(0.05 0.05 0.05 1);
		 translationX: width*-1 y:(2*width) z: width;
		 pitchYawRoll: 110@135@0;
		 solid: false.
	lights add: light.
	self addChild: light.
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/2/2009 09:28'!
placeMirrors: specs

	specs do: [ :spec | 
		| mirror trans |
		trans := self panelTranslationFromSpec: spec.

		"A spec is (x@y@z) . yaw Degrees."

		mirror := QMirror new.
		mirror extent: 7@5.
		mirror translation: trans.
		mirror yaw: spec second.
		mirror isPlacementLocked: true.
		self addChild: mirror.
	].! !

!QConferenceRoom methodsFor: 'factory' stamp: 'sn 3/9/2013 23:37'!
placeNoteboard: spec

	| panel trans dest |

	trans := self panelTranslationFromSpec: spec.

	"A spec is (x@y@z) . yaw Degrees. extent "

	panel := QNoteBoard new.
	panel extent: spec third.
	panel objectName: 'Note Board'.

	dest := QDestination consistingOf: panel.
	dest
		translation: trans;
		yaw: spec second;
		category: 'Named Places';
		destinationMode: #front;
		buttonLabel: 'Notes and Topics'translateMe;
		objectName: 'Notes and Topics';
		suggestedPOV: 'First Person';
		buttonPosition: (0 @ (spec third y / 2.0 + 1.5) @ 0).
		
	dest isPlacementLocked: true.
	self addChild: dest.

	^ panel! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/4/2008 12:19'!
placeNoteboards: specs

	specs do: [ :spec | 
		self placeNoteboard: spec.
	].	
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/1/2009 21:48'!
placePedestals: specs
	"Make  3D pedestals for the room."
	specs do: [:spec |
		| edge pedestal |
		edge := 5.5.	"The length of a side of the portal cube."
		pedestal := (QPedestal new)
			extent: (edge+1)@0.7@(edge+1);
			label: '3D Projection';
			translation: spec first;
			rotationAroundY: (spec at: 2 ifAbsent:[0]);
			isPortalVisible: false;
			isTemplateExclusion: true;
			yourself.
		self addChild: pedestal.
	].
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/6/2007 15:25'!
placeRolodexes: specs

	"The rolodexes are given as x-y plan location, stand height, and rotation"
	specs do: [ :spec |
		| rolodex |
		rolodex :=  QStandRolodex new.
		rolodex basicExtent: 1.5@1.
		rolodex translation: (spec first x@0.05@spec first y);
				height: spec second;
				rotationAroundY: spec third.
		self addChild: rolodex.
	].! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 6/1/2009 21:47'!
placeTables: specs
	"QTables - spec is { location . yaw  . <optional extent>} "

	specs do: [:spec |
		|desk|
		desk := QTable new.
		desk extent: ((spec size > 2) ifTrue: [spec third] ifFalse:[8.2@2.7@4]).
		desk yaw: spec second.
		desk translation: spec first.
		desk isTemplateExclusion: true.
		self addChild: desk.
	].! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 4/8/2007 14:23'!
placeWallFrom: first to: second height: h material:m texture: t repeat: uvs

	"First and second are points where x is x and y is spatial z;
	 run a wall from first to second."

	| w midpoint |

	w :=  QWall new extent:(second dist: first)@h;
		material: m;
	      texture: t;
		uvScale: uvs.
	midpoint := (first + second) /2.0.
	w translation: midpoint x @ 0 @ midpoint y.
	w rotationAroundY: 90-(first bearingToPoint: second) .
	self addChild: w.

	^ w.! !

!QConferenceRoom methodsFor: 'factory' stamp: 'bgf 9/6/2007 15:26'!
placeWalls: specs height: h material: m

	| w walls |

	walls := OrderedCollection new.
	specs do: [ : spec |
		w := self placeWallFrom: spec first 
			to: spec second 
			height: h
			material: m copy
			texture: (QTextures find: spec third) 
			repeat: (spec fourth).
		walls add: w.
	].
! !

!QConferenceRoom methodsFor: 'factory' stamp: 'sn 3/4/2013 21:31'!
populate: layout

	"World-building entry to assemble the basic room -
	 floor, walls, ceiling, and avatar limits."

	self extent: layout extent.
	avatarBox := layout avatarRange.
	self homeTransform: layout homeTransform.

	self placeLights.
	self placeFloor: layout.
	self placeCeiling: layout.
	self placeWalls: layout walls height: self roomHeight material: layout wallMaterial.
	self placeFans: layout fanLocations.
	self placeMirrors: layout mirrorLocations.
	self placeDisplayPanels: layout panelLocations.
	self placeNoteboards: layout noteboardLocations.
	self placeGates: layout gateLocations.
	self placeFloorPaths: layout floorPaths material: layout floorPathMaterial layout: layout.
	self placeRolodexes: layout rolodexLocations.
	self placeLandmarks: layout landmarks.
	self placePedestals: layout pedestalLocations.
	self placeTables: layout tables.

	layout placeCustomElements.

	QLogger logInfo: 'Forum assembled.'translateMe.! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 1/24/2008 17:31'!
raiseCeiling

	self adjustHeightBy: ((self extent y/6.0) min: 10.0 max: 1.0) .
! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/2/2009 12:38'!
repairProjectForumRug: tframe
	"The rugs in the Project-(Baltic / Caribbean / Pacific) templates
	  cause picking problems; perhaps the fruit of older importer/geometry bugs."

	| rect |
	(tframe class == TGeometry) ifFalse: [ ^ false ].
	
	((tframe objectName = 'PersianRug')
	  or: [ tframe objectName = 'RedRug']
	  or: [ tframe objectName = 'Mod_Rug_Taupe']) ifFalse: [ ^ false ].

	"Let's just tear up that shabby old rug and lay down a nice shiny new one."
	rect := TRectangle new.
	rect material: tframe material.
	rect extent: 22@26.
	rect translation: tframe translation + (tframe boundingBox center).
	rect addRotationAroundX: -90.
	rect addRotationAroundY: 0.4.
	rect objectName: (tframe objectName, '-2').
	rect doContextMenu: false.
	rect isFromTemplate: true.

	self replaceChild: tframe with: rect.
	^ true
! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/1/2009 02:01'!
repairRails: tframe
	"The railings Project-(Baltic / Caribbean / Pacific) templates, need to be template-marked."

	(tframe class == TGroup) ifFalse: [ ^ false ].
	(tframe children size = 2) ifFalse: [ ^ false ].
	(tframe objectName = 'Rails' or: [tframe objectName = 'Railings']) ifFalse: [ ^ false ].
	(tframe children allSatisfy: [ : f | f class = TFrameThumb ]) ifFalse: [ ^ false ].

	tframe isFromTemplate: true.
	tframe doContextMenu: false.
	tframe objectName = 'DeckRails'.	"(Marking the repair.)"

	^ true
! !

!QConferenceRoom methodsFor: 'repair-upgrade' stamp: 'bgf 6/2/2009 14:03'!
repairReceptionFlags: tframe
	"A couple of stray context menu cases."

	(tframe class == TGeometry) ifFalse: [ ^ false ].
	((tframe objectName = 'Mod_Runner_Taupe_1' )
	  or: [ tframe objectName = 'ReceptionCurveRug_1' ]) ifFalse: [ ^ false ].

	tframe doContextMenu: false.

	"Mark this so we don't do it again."
	tframe objectName = 'Mod_Runner_Taupe_1' ifTrue: [
		tframe objectName: ('Reception-Rug').
	] ifFalse: [
		tframe objectName: ('Reception-Curved-Rug').
	].
	^ true! !

!QConferenceRoom methodsFor: 'persist' stamp: 'bgf 6/2/2009 17:31'!
repairSpecialCases
	"Fish out any recognized special case objects 
	 that I have a macro-repair method for, and apply it.
	 This is done when unpickling spaces."

	super repairSpecialCases.

	true caseOf: { 
		[ self checkLibraryTemplateFlags ] -> [].
		[ self checkReceptionTemplateFlags ] -> [].
		[ self checkProjectTemplateFlags ] ->[].
	} otherwise: [].
! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 9/17/2007 11:45'!
resetLights

	"An action verb.  Scour the lights from the space and replace them with defaults."

	(self findInstancesOf: TLight) do: [ : light | light future destroy ].
	lights := OrderedCollection new.
	self placeLights.! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 17:06'!
rolodexes
	^ self frameChildren select: [:c|c isKindOf:QRolodex]! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 1/31/2007 23:18'!
roomDepth
	"Dimension of the room along its Z axis."
	^ extent z
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 1/31/2007 23:18'!
roomHeight
	^ extent y
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 1/31/2007 23:28'!
roomWidth	
	"Dimension of the room along its X axis"
	^ extent x
! !

!QConferenceRoom methodsFor: 'initialize' stamp: 'hrs 3/14/2009 10:58'!
spawnAutoSaveJob
	"Spawn the auto-save loop for this space if one is requested"

	^ QAutoSaveJob defaultForForumSpace: self! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 3/9/2007 17:11'!
toggleChild:aChild

	(aChild hasAncestor: self) ifTrue: [
		self removeChild: aChild.
	] ifFalse: [
		self addChild: aChild.
	].! !

!QConferenceRoom methodsFor: 'actions' stamp: 'bgf 4/7/2007 20:09'!
toggleMainLights

	lights size > 0 ifTrue: [self toggleChild: lights first].
	lights size > 1 ifTrue: [self toggleChild: lights second].
	! !

!QConferenceRoom methodsFor: 'initialize' stamp: 'bgf 11/1/2006 21:07'!
uiFormatterClass 

	"The formatter adjusts nav panel readout for local space coordinate conventions."
	^#QDefaultUIFormatter.
! !

!QConferenceRoom methodsFor: 'accessing' stamp: 'bgf 3/9/2007 16:55'!
walls
	"All the QWalls in the space."
	^ self find: [:c|c isKindOf:QWall]! !

QSpace subclass: #QForumNaCl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums'!

!QForumNaCl class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumNaCl methodsFor: 'objects' stamp: 'bgf 7/17/2007 23:52'!
addGate: aSymbol label: aString

	"Add a gate portal into the space, placing it on the ground.
	 The gate portal is registered under the given global symbolfor inbound linking).

	 Note that the result will have to be translated into a new position
	 if more than one gate is needed."

	| gate |
	gate := (QGate new)
			translation: (self adjustToGround:(5@0@-12));
			label: aString
			yourself.
	self addChild: gate.
	gate registerGlobal: aSymbol.
	^ gate! !

!QForumNaCl methodsFor: 'initialize' stamp: 'das 2/25/2007 13:28'!
adjustToGround: trans

	^ trans.! !

!QForumNaCl methodsFor: 'initialize' stamp: 'das 2/27/2007 21:05'!
initialize
	super initialize.
	self registerGlobal: #NaCl.
! !

!QForumNaCl methodsFor: 'initialize' stamp: 'das 2/25/2007 13:54'!
makeLights: space

	| light |

"warm light in northwest corner."
	light := TLight new.
	light ambientColor: #(0.3 0.4 0.5 0.6).
	light diffuseColor: #(0.48 0.35 0.4 0.6).
	light specularColor: #(0.3 0.23 0.2 0.5).
	light rotationAroundY: -130.
	light addRotationAroundX: 120.
	space addChild: light.

"cool light diagonally opposite."
	light := TLight new.
	light ambientColor: #(0.1 0.1 0.1 0.1).
	light diffuseColor: #(0.4 0.35 0.48 0.6).
	light specularColor: #(0.2 0.23 0.3 0.5).
	light rotationAroundY: 132.
	light addRotationAroundX: 120.
	space addChild: light.

" fainter green light in third corner."
	light := TLight new.
	light ambientColor: #(0.1 0.1 0.1 0.1).
	light diffuseColor: #(0.1 0.23 0.1 0.4).
	light specularColor: #(0.3 0.2 0.3 0.3).
	light rotationAroundY: 42.
	light addRotationAroundX: 120.
	space addChild: light.

"and another opposite it"
	light := TLight new.
	light ambientColor: #(0.1 0.1 0.1 0.1).
	light diffuseColor: #(0.1 0.23 0.1 0.4).
	light specularColor: #(0.4 0.3 0.2 0.2).
	light rotationAroundY: -42.
	light addRotationAroundX: 120.
	space addChild: light.
! !

!QForumNaCl methodsFor: 'objects' stamp: 'bgf 9/2/2007 11:12'!
makeSalt: len scale: scl

	| naCl spin |
	
	naCl := QSaltCrystal new.
	naCl length: len; 
		 spacing: scl.

	spin := TSpinner new.
	spin matNil.
	spin contents: naCl.
	spin translation: 4.5@4@20.
	self addChild: spin.! !

!QForumNaCl methodsFor: 'initialize' stamp: 'bgf 9/2/2007 11:06'!
populate

	| |
"	pointCloud := TPointCloud new.
	pointCloud sphericalShell:100000.
	self addChild: pointCloud.
"

	self makeSalt:40 scale:0.2.
	self makeLights: self.
! !

Q3DSModel subclass: #QOutdoorTree
	instanceVariableNames: 'treeBase'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QOutdoorTree commentStamp: '<historical>' prior: 0!
A 3DS-Model that adds a tree base cone.
Used for trees in the campus.!

!QOutdoorTree class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QOutdoorTree methodsFor: 'accessing' stamp: 'bgf 9/21/2007 15:17'!
baseOffset: aPoint

	"Adjusts the position of the tree base cone in x-z...
	 use this when the tree trunk of my 'mesh' meets the ground off center."

	treeBase ifNotNil: [
		treeBase translationX: aPoint x y: (treeBase translation y) z: aPoint y.
	]! !

!QOutdoorTree methodsFor: 'persist' stamp: 'bgf 9/21/2007 14:40'!
constructFrom: pStream

	| offset |
	super constructFrom: pStream.
	treeBase ifNotNil: [
		offset := pStream readPoint: 'baseOffset' default: 0@0.
		treeBase translationX: offset x y: (treeBase translation y) z: offset y.
	].! !

!QOutdoorTree methodsFor: 'initialize' stamp: 'bgf 9/21/2007 14:35'!
initialize
	super initialize.! !

!QOutdoorTree methodsFor: 'initialize' stamp: 'bgf 9/21/2007 15:19'!
installedMesh: tree

	"Once I have a mesh, I can add its root-bump base cone..."
	treeBase := QTreeBase new.
	treeBase setSizeForTree: tree.
	self addChild: treeBase.! !

!QOutdoorTree methodsFor: 'persist' stamp: 'bgf 9/21/2007 14:38'!
persistOn: pStream
	super persistOn: pStream.

	treeBase ifNotNil: [
		pStream storePoint: 'baseOffset' value: (treeBase translation x @ treeBase translation z) default: 0@0.
	].! !

!QOutdoorTree methodsFor: 'accessing' stamp: 'bgf 9/21/2007 14:36'!
treeBase
	^ treeBase! !

QDisplayPanel subclass: #QPeeringDisplayPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QPeeringDisplayPanel commentStamp: '<historical>' prior: 0!
A leftover, held in the system so old forums that refer to it in persistence can be recovered.!

!QPeeringDisplayPanel class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

Object subclass: #QForumConverter
	instanceVariableNames: 'template rolodexes pedestals gates panels posters signs notes models'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums'!

!QForumConverter commentStamp: '<historical>' prior: 0!
Rebuilds a forum to the current definition of any hard-coded template class.

Highly experimental, incomplete, and generally suspect.
Does a basic job of copying over content.

Does not add display panels or anything to fit extra content;
surplus (file-card) content is dumped into an overflow rolodex.

Notes, signs, and posters are dumped on the floor in piles.

Inbound links to gates will probably be lost,
unless they're links to the primary (named) entry portal.


!

!QForumConverter class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumConverter class methodsFor: 'utilities' stamp: 'bgf 9/10/2007 11:18'!
templatesForSpace: aSpace service: aServiceRecord client: aParticipant

	"Return the list of template QServiceDescriptions that can be used to upgrade the given space.
	 Currently we offer to upgrade any conference room to any kind of room template,
 	 using the template maker."

	(((aSpace get:#class) ifNil: [Object]) includesBehavior: QConferenceRoom) ifFalse: [
		^ #()
	].

	^ (((QServiceDescription allForClient: (aParticipant class)) asArray) 
		select: [ : desc | desc entryWorld includesBehavior: QForumTemplate ])
	   sortBy: [ :d1 :d2 | d1 name < d2 name ]
	   ! !

!QForumConverter methodsFor: 'private' stamp: 'bgf 9/8/2007 14:45'!
annotationSpotFor: record atRadius: radius

	"Return a point on the circle of given radius from the room origin,
	 along the same bearing as the global position when the note was recorded.
	 If the original note was closer to the center of the forum, we respect that.

	 We don't preserve the exact location	
	 because forum sizes vary, and all the content is shifted into new canonical positions. 
 	 So it is clearer to leave the notes laid out 'near' where they were before,
	 laid out on the floor so you can easily see and restore them,
	 rather than pretending to get them back into precise positions."

	| point r |

	r := (radius min: (record first x @ record first z) length) max: 3.0.
	point := record first x @ record first z.
	^ point * (r / (point length max: 1)).

! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/6/2007 22:56'!
captureCardStands: aSpace

	"Record all the stand-rolodexes in the space."
	( aSpace findInstancesOf: QStandRolodex) do: [ : dex |
		dex size > 0 ifTrue: [
			self rolodexes add: {dex globalPosition . dex duplicate}.
		].
		dex destroy.
	].! !

!QForumConverter methodsFor: 'upgrade' stamp: 'bgf 9/4/2007 19:57'!
captureContent: aSpace

	"Grab all the 'data' we can handle from the old forum.
	 These operations are destructive, removing the original as they go."

	self captureNotes: aSpace.
	self captureSigns: aSpace.
	self capturePosters: aSpace.

	self capturePanels: aSpace.
	self capturePedestals: aSpace.
	self captureGates: aSpace.
	self captureCardStands: aSpace.

	self captureLeftoverCards: aSpace.
	self captureLeftoverModels: aSpace.! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/22/2007 01:06'!
captureGates: aSpace

	( aSpace findInstancesOf: QGate) do: [ : gate |

		gate postcardLink ifNotNil: [
			self gates add: { 
				gate globalPosition .
				gate postcardLink urlString. 
				gate label .
				gate isOpen .
				gate returnFrame globalName }.
		].
		gate destroy.
	].
! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/6/2007 23:29'!
captureLeftoverCards: aSpace

	"Record all the file cards from rolodexes remaining..."
	( aSpace findInstancesOf: QRolodex) do: [ : dex |
		dex isInAvatar ifFalse: [
			self rolodexes add: { dex globalPosition . dex duplicate.}.
			dex destroy.
		].
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/4/2008 12:02'!
captureLeftoverModels: aSpace

	| modelClasses |
	modelClasses := { 
		TGeometry . Q3DSModel . QPortableObject . QFixedSurface . QEditBox .
		TSphere . TConic . TCube . TAnimatedFigureModel . TSkeleton . TFrameThumb }.

	"NB - this misses primitive-objects using TCube like the Cube, Slab, etc from the objects menu...
	 not quite sure how to filter for those without including old floor and ceiling,
	 which were originally just cubes in old forums."

	aSpace children do: [ : kid | 
		((kid class = TGroup and: [ kid children notEmpty]) 
		 or:  [modelClasses includes: kid class] )
		ifTrue: [
			"The floor was a named cube in earlier forums..."
			(kid objectName = 'floor') ifFalse: [
				models add: kid.
				aSpace removeChild: kid.
			].
		].
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/8/2007 13:41'!
captureNotes: aSpace

	(aSpace findInstancesOf: QStickyNote) do: [ : sticky | 
		self notes add: { sticky globalPosition . sticky contents }.
		sticky destroy.
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/8/2007 13:25'!
capturePanels: aSpace

	"Record all the display panel content in the space."

	(aSpace findInstancesOf: QDisplayPanel) do: [ : panel |
	
		(panel contentCard content notNil or: [
		 panel historyRolodex size > 1]) 
		ifTrue: [
			self panels add: {
				panel globalPosition copy.
				panel contentCard duplicate .
				panel historyRolodex duplicate .
				panel isFadeEnabled .
			}.
		].
		panel destroy.
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/8/2007 12:58'!
capturePedestals: aSpace
	"Record all the display panel content in the space."

	( aSpace findInstancesOf: QPedestal) do: [ : pedestal |
		pedestal postcardLink ifNotNil: [
			self pedestals add: {
				pedestal globalPosition copy.
				pedestal postcardLink ifNotNilDo: [ : pc | pc urlString ] .
				pedestal label .
				pedestal initialScale .
				pedestal initialOffset .
				pedestal isPortalVisible . 
			}.
		].
		pedestal destroy.
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/8/2007 13:41'!
capturePosters: aSpace

	(aSpace findInstancesOf: QPoster) do: [ : poster |
		self posters add: {
			poster globalPosition .
			poster contents . 
			poster extent }.
		poster destroy.
	].! !

!QForumConverter methodsFor: 'capture' stamp: 'bgf 9/8/2007 13:41'!
captureSigns: aSpace

	(aSpace findInstancesOf: QDisplaySign) do: [ : sign |
		self signs add: { sign globalPosition . sign contents . sign extent }.
		sign destroy.
	].
! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
gates
	^ gates! !

!QForumConverter methodsFor: 'private' stamp: 'bgf 9/8/2007 12:14'!
headingOfItem: aFrameOrSpec

	"The heading (angle made with the origin) of the given TFrame or template item spec;
	 template item specs are lists whose first entry is the position."

	| p loc |
	(aFrameOrSpec isKindOf: TFrame) ifTrue: [

		p := aFrameOrSpec globalPosition.
		^ (p x @ p z) degrees

	] ifFalse: [
		loc := aFrameOrSpec first.
		(loc isPoint) ifTrue: [
			^ loc degrees
		].
		 (loc size = 2) ifTrue: [ 
			^ (loc first @ loc second) degrees
		 ] ifFalse: [ 
			^ (loc first @ loc third) degrees
		].
	].
! !

!QForumConverter methodsFor: 'private' stamp: 'bgf 9/8/2007 13:17'!
indexNearestHeading: vec3 from: sortedList

	"The list may contain both TFrames and spec-lists;
	 the first entry of the spec list is used as its position."

 	| delta index heading |
	delta := Float infinity.
	index := 0.
	heading := (vec3 x @ vec3 z) degrees.
	sortedList doWithIndex: [ :item :i |
		| d | 
		d := (heading - (self headingOfItem: item)) abs.
		d > (delta + Float epsilon) ifTrue: [
			"Cutoff - past the point of interest."
			^ index
		] ifFalse: [
			delta := d.
			index := i.
		].
	].
	^ index
			! !

!QForumConverter methodsFor: 'initialize-release' stamp: 'bgf 9/4/2007 18:10'!
initialize
	
	"Each of these is a list of spec-data describing content we want to carry over."

	panels := OrderedCollection new.
	rolodexes := OrderedCollection new.
	gates := OrderedCollection new.
	pedestals := OrderedCollection new.
	models := OrderedCollection new.
	posters := OrderedCollection new.
	signs := OrderedCollection new.
	notes := OrderedCollection new.! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
models
	^ models! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:55'!
notes
	^ notes! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
panels
	^ panels! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
pedestals
	^ pedestals! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
posters
	^ posters! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/9/2007 20:35'!
restoreCardStands: aSpace overflow: overflow

	| targets |

	targets := (aSpace findInstancesOf: QStandRolodex).

	targets := self sortedByHeading: targets.
	rolodexes := self sortedByHeading: rolodexes.
	rolodexes do: [ : record |
		| i |
		i := self indexNearestHeading: (record first) from: targets.
		i > 0 ifTrue: [
			"Got a rolodex from the supply."
			record second cards do: [ :cd | (targets at: i) insertCard: cd ].
			targets removeAt: i.
		] ifFalse: [	
			"Toss the leftovers into the overflow dex ..."
			record second cards do: [ :cd | overflow insertCard: cd ].
		].
	].
! !

!QForumConverter methodsFor: 'upgrade' stamp: 'bgf 9/9/2007 20:09'!
restoreContent: aSpace

	"Try to stuff the content back into the premade facilities in the space.
	 Anthing without an obvious fit gets dropped into the 'overflow' rolodex."

	| overflow |

	overflow :=  QStandRolodex new.
	overflow basicExtent: 1.5@1.0;
			 translation: (-3@0@-3);
			 height: 4.0.
	aSpace addChild: overflow.

	self restorePanels: aSpace overflow: overflow.		"Does posters also."
	self restoreGates: aSpace overflow: overflow.
	self restorePedestals: aSpace overflow: overflow.
	self restoreCardStands: aSpace overflow: overflow.

	self restoreSigns: aSpace.
	self restoreNotes: aSpace.
	self restoreLeftoverModels: aSpace.

	overflow cards ifEmpty: [ overflow destroy ].! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/8/2007 12:53'!
restoreGate: gate fromRecord: rec

	| pc |
	pc := rec second ifNotNil: [ TPostcard urlString: rec second ].
	gate postcardLink: pc; 
		label: rec third;	
		isOpen: rec fourth.

	rec fifth ifNotNilDo: [ : gn | 
		gate registerGlobal: (gn asSymbol).
	].
	! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 1/2/2008 23:49'!
restoreGates: aSpace overflow: overflow

	| available item pc |

	"Make a heterogenous list - gates the template has already made,
	 and template-specs for extra gate locations."
	available := (self sortedByHeading: (
						(aSpace findInstancesOf: QGate), 
						(template extraGateLocations))).
	gates := self sortedByHeading: gates.
	
	"Preserve grouping by keeping the 'o-clock' ordering the same."
	gates do: [ :record |
		| i |
		i := self indexNearestHeading: (record first) from: available.
		i > 0 ifTrue: [
			"Got a panel (or a template-spec for adding one)"
			item := available at: i.
			(item isKindOf: TFrame) ifFalse: [	
				"If not gate, then it is a spec for an extraGateLocation"
				item := (aSpace placeGate: item).
			].
			self restoreGate: item fromRecord: record.
			available removeAt: i.
		] ifFalse: [
			"Out of spots - dump it in the overflow 'dex."
			pc := record second ifNotNil: [ TPostcard urlString: record second ].
			pc ifNotNil: [
				overflow insertCard: (QFileCard forPostcard: pc).
			].
		].
	].
! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/4/2007 20:08'!
restoreLeftoverModels: aSpace

	self models do: [ : kid |
		aSpace addChild: kid.
	].! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/8/2007 14:49'!
restoreNotes: aSpace

	| spot radius |

	"We lay them out in a circle around the origin."

	radius := ((template extent x min: template extent z) / 3.0) max: 6.0.

	self notes doWithIndex: [ : record : i |
		| note | 
		note := (QStickyNote new)
					contents: record second;
					yourself.

		spot := self annotationSpotFor: record atRadius: radius.
	
		note turnToFaceNormal: (0@1@0).
		note addRotationAroundZ: (90 - spot degrees).

		spot := (aSpace adjustToGround: spot) + (0@0.4@0).	"Above posters and notes."
		note translation: spot.
		aSpace addChild: note.
	].
! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/8/2007 12:52'!
restorePanel: panel fromRecord: aRecord

	panel displayCard: aRecord second.
	panel isFadeEnabled: aRecord fourth.
	panel historyRolodex ifNotNilDo: [ : h |
		(aRecord third) cards 
			reject: [:cd | cd = panel contentCard]
			thenDo: [ :cd | h insertCard: cd duplicate ].
	].! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/9/2007 20:36'!
restorePanels: aSpace overflow: overflow

	| available item |

	"Make a heterogenous list - panels the template has already made,
	 and template-specs for extra panel locations."
	available := (self sortedByHeading: (
						(aSpace findInstancesOf: QDisplayPanel), 
						(template extraPanelLocations))).
	panels := self sortedByHeading: panels.
	
	"Preserve grouping by keeping the 'o-clock' ordering the same."
	panels do: [ :record |
		| i |
		i := self indexNearestHeading: (record first) from: available.
		i > 0 ifTrue: [
			"Got a panel (or a template-spec for adding one)"
			item := available at: i.
			(item isKindOf: TFrame) ifFalse: [
				"If not panel, then it is a spec for an extraPanelLocation"
				item := (aSpace placeDisplayPanel: item).
			].
			self restorePanel: item fromRecord: record.
			available removeAt: i.
		] ifFalse: [
			"Out of spots - dump its history in the overflow 'dex."
			(record third) cards do: [ :cd | overflow insertCard: cd duplicate ].
		].		
	].

	"Now put the posters back using the remaining empty panel slots."

	available := available reject: [ : entry | entry isKindOf: TFrame ].
	self restorePosters: aSpace usingPanelPlaces: available.! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 1/2/2008 23:49'!
restorePedestals: aSpace overflow: overflow

	| targets pc |

	targets := self sortedByHeading: (aSpace findInstancesOf: QPedestal).
	pedestals := self sortedByHeading: pedestals.

	pedestals do: [ :record |
		| i |
		pc := record second ifNotNil: [ TPostcard urlString: record second ].
		i := self indexNearestHeading: (record first) from: targets.
		i > 0 ifTrue: [
			"Got a pedestal."
			(targets at: i) 
				postcardLink: pc; 
				label: record third;
				initialScale: record fourth;	
				initialOffset: record fifth;	
				isPortalVisible: record sixth.

			targets removeAt: i.
		] ifFalse: [
			pc ifNotNil: [
				overflow insertCard: (QFileCard forPostcard: pc).
			].
		].	
	].! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/9/2007 20:40'!
restorePosters: aSpace usingPanelPlaces: available

	| spot index spec |

	"Place the posters in the remaining available panel positions
	 (ones that don't have display panels),
	 and  pile leftovers in the middle of the room.
	 (Note that available should contain only template panel specs,
	 not display panel frames.)"

	posters := self sortedByHeading: posters.
	self posters doWithIndex: [ :record : i |
		| sign | 
		sign := (QPoster new)
					contents: record second;
					extent: record third;
					yourself.

		index := self indexNearestHeading: record first from: available.
		index > 0 ifTrue: [

			spec := available at: index.
			sign translation: (aSpace panelTranslationFromSpec: spec).  "Handles wall thickness."
			sign yaw: spec second.
			available removeAt: index.
	
		] ifFalse: [
			"Leave it laying on the ground."
			spot := self annotationSpotFor: record atRadius: 10.0.
			spot := (aSpace adjustToGround: spot) + (0@(0.15 +(0.005*i))@0).
			sign turnToFaceNormal: (0@1@0).
			sign translation: spot.
		].
		aSpace addChild: sign.
	].
! !

!QForumConverter methodsFor: 'restore' stamp: 'bgf 9/8/2007 14:49'!
restoreSigns: aSpace

	| spot |

	"We just pile signs up in the middle of the room."

	self signs doWithIndex: [ :record : i |
		| sign | 
		sign := (QDisplaySign new)
					contents: record second;
					extent: record third.

		spot := self annotationSpotFor: record atRadius: 8.0.
		spot := (aSpace adjustToGround: spot) + (0@0.25@0).	"On top of posters."

		sign turnToFaceNormal: (0@1@0).
		sign translation: spot.
		aSpace addChild: sign.
	].
! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:53'!
rolodexes
	^ rolodexes! !

!QForumConverter methodsFor: 'accessing' stamp: 'bgf 9/4/2007 14:54'!
signs
	^ signs! !

!QForumConverter methodsFor: 'private' stamp: 'bgf 9/8/2007 11:52'!
sortedByHeading: aList

	^ aList asOrderedCollection sortBy: [ :item1 :item2 | 
		 (self headingOfItem: item1) < (self headingOfItem: item2)
	].! !

!QForumConverter methodsFor: 'upgrade' stamp: 'bgf 5/31/2009 17:46'!
upgradeSpace: space toTemplate: aTemplateClass

	self captureContent: space.

	space frameChildren copy do: [ :fc |
		(fc objectName = 'defaultMainEntry' or: [ fc isInAvatar ]) ifFalse: [
			fc destroy
		].
	].
	space populate: (template := aTemplateClass newForUpgrade: space).
	space markAsTemplateInstance.

	self restoreContent: space.! !

BaseWorld subclass: #QFlatCampus
	instanceVariableNames: 'space'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QFlatCampus class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QFlatCampus class methodsFor: 'services' stamp: 'bgf 9/4/2008 12:54'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Campus (Lawn)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI
			! !

!QFlatCampus class methodsFor: 'services' stamp: 'bgf 10/1/2007 12:33'!
islandName
	^ 'Lawn'! !

!QFlatCampus methodsFor: 'initialize-release' stamp: 'bgf 6/1/2009 21:58'!
initialize
	space := QForumGarden new.
	space generateFlatTerrain;
		   populate;
		   markAsTemplateInstance;
		   registerDefaultMainEntry.! !

BaseWorld subclass: #QForumCampus
	instanceVariableNames: 'space'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QForumCampus commentStamp: '<historical>' prior: 0!
World-builder for generating a empty campus (forum garden).

The campus will receive only trees; gates to particular forums are added later.!

!QForumCampus class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumCampus class methodsFor: 'services' stamp: 'bgf 9/4/2008 12:54'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Campus (Hills)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI
			! !

!QForumCampus class methodsFor: 'services' stamp: 'bgf 3/7/2007 14:58'!
islandName
	^ 'Garden'
	"So the default service name 'campus' gives local names 'Campus/Garden'"! !

!QForumCampus methodsFor: 'initialize-release' stamp: 'bgf 6/1/2009 21:57'!
initialize

	space := QForumGarden new.
	space generateFractalTerrain.		"Ground goes first, so height-above-ground works."
	space populate;
		   markAsTemplateInstance.
	space registerDefaultMainEntry.! !

BaseWorld subclass: #QForumTemplate
	instanceVariableNames: 'space'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums'!

!QForumTemplate commentStamp: '<historical>' prior: 0!
Construction template for a forum conference room.

I largely consist of protocols intended to be overridden by subclasses.

The base methods here provide the medium-sized conference room
in the wood+white appearance.   Minimal content is included
(web links to google and Qwaq, the Qwaq presentation movie, 
and a pedestal link to the forum itself.)

My 'layout' category describes the structure and furnishings.

My 'content' category describes the initial content,
such as graphics, applications, links to other spaces.!

QForumTemplate subclass: #QExtraRoomTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QExtraRoomTemplate commentStamp: '<historical>' prior: 0!
Vestigial default template for side rooms.
Present only to satisfy persisted references to this class;
the add-side-room functions have been removed.
!

!QExtraRoomTemplate class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QExtraRoomTemplate methodsFor: 'layout' stamp: 'bgf 1/26/2008 17:14'!
extent
	"Secondary rooms are smaller."
	^ 65@12@85! !

!QExtraRoomTemplate methodsFor: 'layout' stamp: 'bgf 4/27/2007 21:14'!
pedestalLocations
	"none"
	^ { }! !

!QForumTemplate class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumTemplate class methodsFor: 'defaults' stamp: 'bgf 3/10/2007 00:15'!
initialEditableText

^
'*** COLLABORATIVE EDITING ***

This text can be edited here in the virtual space.

As edits are made, both the changes and the activity of editing are visible to all participants.

One participant at a time holds the input focus for the editor; when you are done editing, you can release it by moving your mouse away from the window.'! !

!QForumTemplate class methodsFor: 'defaults' stamp: 'ar 4/16/2011 17:12'!
islandName
	^ 'Forum'! !

!QForumTemplate class methodsFor: 'instance creation' stamp: 'bgf 9/4/2007 13:57'!
newForUpgrade: aSpace
	^ (self basicNew) initializeForUpgrade: aSpace! !

!QForumTemplate class methodsFor: 'defaults' stamp: 'bgf 9/3/2007 20:49'!
sisalCarpet

	"For reference - the textured tan carpet we use a lot of."
	^  (TMaterial new)
		color: (Color r:0.64 g: 0.61 b: 0.56);
		texture: (QTextures find: ('carpetTan.jpg'))! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 5/14/2007 23:14'!
avatarRange
	"Return a TBox describing the limits of avatar motion."
	^ space normalAvatarBox! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 7/14/2007 15:39'!
ceilingMaterial

	"Default material is formed from default texture file, 
	 on a light gray material.  A nil result here suppresses the ceiling."	

	self ceilingTextureFile ifNil: [ ^ nil ].

	^  (TMaterial new)
		color: (Color r:0.98 g:0.96 b:0.92);
		texture: (QTextures find: (self ceilingTextureFile))

! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 7/14/2007 13:55'!
ceilingTextureFile

	^ 'ceilingBroad.jpg'! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 7/14/2007 17:41'!
ceilingUVScale

	^ 12@14! !

!QForumTemplate methodsFor: 'content' stamp: 'bgf 4/27/2007 21:01'!
entryDescription

	"If supplied, is used define the postcard link to the main entry portal
	 List of 3 elements: label,  URL-String, and flag:isInitiallyOpen.
	 For example: 

		{ 'Gulf of Mexico' . '../, MeredithWorld islandName, '/QwaqPortal', false }

	This is usually left nil, so that on-the-fly linking finds the main portal free.	"

	^ nil! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 1/28/2008 14:11'!
extent
	"Return a Vector3 describing the outermost floor dimensions,
	 and the height of the ceiling"

	^ 96@12@72! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/6/2007 15:27'!
extraGateLocations

	"Gate locations when extra gates are needed when renovating."

	| |

	^ #()
! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/8/2007 14:27'!
extraPanelLocations
	"The ordered list of surplus display panel positions used only when renovating.
	{ x@y@z . rotationDegrees }
	"
	| height width depth |

	height := 5. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"outer ends of north and south..."
		{ 3.25*width/-8.0@height@(depth / -2.0) . 0 } .
		{ 3.25*width/8.0 @ height @ (depth/-2.0) . 0 } .
		{ 3.25*width/8.0 @ height @ (depth/2.0) . 180 } .
		{ 3.25*width/-8.0 @ height @ (depth/2.0) . 180 } .

		"inward of east and west walls"
		{ width/2.0 @ height @ (depth/-5) . -90 } .
		{ width/2.0 @ height @ (depth/5) . -90 } .
		"west wall" 
		{ width/-2.0 @ height @ (depth/-5) . 90 } .
		{ width/-2.0 @ height @ (depth/5) . 90 } .

		"inner parts of north."
		{ width/-5.8@height@(depth / -2.0) . 0 } .
		{ width/5.8 @ height @ (depth/-2.0) . 0 } .
		"inner parts of south."
		{ width/-5.8@height@(depth / 2.0) . 180 } .
		{ width/5.8 @ height @ (depth/2.0) . 180 } .
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/5/2007 01:34'!
fanLocations

	"{ x@y . rotationDegrees }  - floor-standing fan positions. "

	^ { 	
		{ -6.3@(self extent z/-2 + 1.1). 0 }
	}! !

!QForumTemplate methodsFor: 'initialize' stamp: 'bgf 8/19/2009 12:43'!
fillWithContent
	
	"Fill the room with rolodex cards, posters, and a link for the 3d podium."

	| posters postcard rolocards entryData|

	"Rolodex contents for the rooms, in order of room creation.
	 Entries are: rolodex number (1-4), content, optional face."
	
	rolocards := self rolodexCards.
	rolocards do: [:entry | 
		space addCard: entry second toRolodex: (space rolodexes at: entry first).
	].

	"Wall posters in shades - shade number, then Qwaq texture file,
	 then optional initial title string."
	posters := self posters.
	posters do: [:pair |
		(space displays at: pair first) displayCard: (space makeContent: pair second).
		pair size > 2 ifTrue: [
			(space displays at: pair first) title: pair third.
		].
		pair size > 3 ifTrue: [
			(space displays at: pair first) isFadeEnabled: pair fourth
		].
	].

	"The content of each room's main 3D portal podium. Each entry contains:
	   { IslandName, entryPointName, scale, offset }. 
	The first entry also contains sublists describing the left and right hand side portals."
	self pedestals doWithIndex: [:pdesc :i | 
		pdesc ifNotNil: [
			postcard := TPostcard urlString: '../', pdesc first,'/', pdesc second.
			(space pedestals at:i) 
				postcardLink: postcard;
				initialScale: pdesc third;
				initialOffset: pdesc fourth;
				label: pdesc fifth.
		].
	].
	
	"And hook up the front door."
	entryData := self entryDescription.
	entryData ifNotNil: [
		space linkEntryPortal: (TPostcard urlString: entryData second) 
			   label: entryData first.
		entryData third ifFalse: [
			space entryGate closeContents.
		].
	].! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/21/2007 23:08'!
floorMaterial

	"Default material is formed from default texture file, 
	 on a light gray material.  A nil result here suppresses the floor."	

	self floorTextureFile ifNil: [ ^ nil ].

	^  (TMaterial new)
		color: (Color r:0.6 g: 0.58 b: 0.54);
		texture: (QTextures find: (self floorTextureFile))

! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 5/1/2007 15:49'!
floorPathMaterial

	"The material applied to the sections of distinguished floor defined by #floorPaths "

	| mat |
	mat := self floorMaterial.
	mat color: (mat diffuseColor asColor darker asVectorColor).
	^ mat! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 5/1/2007 23:52'!
floorPathUVScale: spec

	"The UVScale for a single strip of floor path;
	 the strip specs come from #floorPaths,
	 and each is (location, extent, rotation).
	By default we assume the floor paths use the same texture as the floor"

	^(self floorUVScale x * (spec second y / self extent z)) @
	   (self floorUVScale y * (spec second x / self extent x)).! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 5/1/2007 15:48'!
floorPaths

	"Returns a spec array,
	 where each spec is  { location-x@y . extent-x@y . rotation-degrees }.
	 These describe areas of floor that are decorated with slightly raised
	 regions that use floorPathMaterial rather than floorMaterial."

	^ #()! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 5/1/2007 15:46'!
floorPathsToGates

	"Generate #floorPaths descriptions leading from
	 the room center to each gate.   Works well only for orthogonal rooms."

	| result  |

	result := OrderedCollection new.

	"A square in the center to meet all the strips"
	result add: { 0@0 . 10@10  . 0 }.

	self gateLocations do: [ : spec |
		| rotation xadj yadj |
		rotation := spec first degrees negated.
		yadj := (rotation degreeSin * 5).
		xadj := (rotation degreeCos * 5).
		result add: {
			((spec first x + xadj)/2@((spec first y - yadj)/2)) .
			(spec first length - 5) @ 10.0 .
			rotation 
		}
	].

	^ result

		


		! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 4/8/2007 14:02'!
floorTextureFile

	^ 'carpetTan.jpg'! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/1/2007 18:52'!
floorUVScale

	"Default uv scale, used for the sisal floor."
	^ self extent x/3.0@(self extent z/3.0) ! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 3/16/2007 16:13'!
gateLocations

	"The ordered list of initial gate locations; 
	 the first listed gate is the main entry (from the garden, or precedeing room in a chain).
	 The second listed gate is the exit gate, which is used to link a subsequent room in this chain."

	| |

	^ { 
		{ 0@(self extent z/2) . 180 . #EntryPortal} .
		{ (self extent x/-2)@0 . 90 } .
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'jdm 8/21/2007 08:16'!
homeTransform
	"The default landing space when logging / teleporting into the space."
	^ Matrix4x4 new translation: -1@5.5@10.! !

!QForumTemplate methodsFor: 'content' stamp: 'bgf 4/27/2007 21:01'!
initialEditableText

	"This method returns the text paragraph that appears in the collaborative editor;
	 it's a simple multiline text string."

	^ self class initialEditableText.! !

!QForumTemplate methodsFor: 'initialize' stamp: 'ar 4/16/2011 17:12'!
initialize
	space := QConferenceRoom new.
	space populate: self.
	space markAsTemplateInstance.

	"The portal links, images and rolodex cards are populated room-by-room."
	self fillWithContent.
	QLogger logInfo: 'Forum populated'.! !

!QForumTemplate methodsFor: 'initialize' stamp: 'bgf 9/4/2007 16:32'!
initializeForUpgrade: aSpace

	"Alternate initializer for forum space upgrade/conversion."
	space := aSpace.! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 8/23/2008 19:01'!
landmarks

	"Fixed initial landmarks for the room;
		{ 'Label' . location . direction . sorting-hint }
	"

	^ {
		{'Bird''s Eye' . 60@70@60 . -45@45@0 . 'b' }
	}
! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 10/4/2007 11:15'!
mirrorLocations
	"The ordered list of QMirror positions;
	{ x@y@z . rotationDegrees }
	"
	| height depth |
	height := 5. "= QPlainAvatar's height above ground."
	depth := self extent z.

	^ { 
		"north wall"
		{ 0 @ height @ (depth/-2.0) . 0 }.
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/4/2008 12:20'!
noteboardLocations
	"The ordered list of brainstorming / note walls
		{ x@y@z . rotationDegrees . extentPoint}
	"
	| height width depth |
	height := 5.45. " ~ avatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"north wall"
		{ width/3.45 @ height @ (depth/-2) . 0 .  13@7.3 } .
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/4/2008 11:36'!
panelLocations
	"The ordered list of display panel positions;
	{ x@y@z . rotationDegrees }
	"
	| height width depth |
	height := 5. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"north wall"
		{ width/-3.45@height@(depth / -2) . 0 } .
		"- slot for mirror -"
		"- slot for brainstorming-" 
		
		"east wall"
		{ width/2.0 @ height @ (depth/-3) . -90 } .
		{ width/2.0 @ height @ 0 . -90 } .
		{ width/2.0 @ height @ (depth/3) . -90 } .

		"south wall" 
		{ width/3.45 @ height @ (depth/2.0) . 180 } .
		 "center reserved for gate"
		{ width/-3.45 @ height @ (depth/2.0) . 180 } .

		"west wall" 
		{ width/-2.0 @ height @ (depth/-3) . 90 } .
		"center reserved for gate"
		{ width/-2.0 @ height @ (depth/3) . 90 } .
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/1/2007 18:16'!
pedestalLocations

	" QPedestals in the room: 
		{ { translation vector . rotationDegrees } ... } "
	^ {
		{ self extent x / 3.9 @ 0 @ (self extent z / 3.9) . -120 } .
		{ self extent x /  3.9 @ 0 @ (self extent z / -3.9) .  -60 } .
	}! !

!QForumTemplate methodsFor: 'content' stamp: 'ar 4/16/2011 17:12'!
pedestals

	"A list of 0, 1 or 2 content descriptors containing links to other spaces, with offset information.
	 These are used to initialize the 3-D display podiums.
	 For instance: 
			{
				{'Underwater World'. #mainEntry. 0.4. 0@0@-8. 'Underwater'}
			}.
	fills a single podium, with a reference to the underwater world.  
	Scale is 0.4; 0@0@-8 is the 3Dportal #offset."

	^ {
		{self islandName . #mainEntry . 0.05 . 0@28@0 . 'This Forum' }.
	}.! !

!QForumTemplate methodsFor: 'extras' stamp: 'bgf 9/4/2007 14:04'!
placeCustomElements

	"Extension point, called after the space is populated with structure,
	 but before content is inserted;
	 the template can insert objects into the space here."

	self placeCustomRugs.! !

!QForumTemplate methodsFor: 'extras' stamp: 'bgf 9/4/2007 14:01'!
placeCustomRugs
	"Placeholder - add carpets to the floor, if desired."! !

!QForumTemplate methodsFor: 'content' stamp: 'bgf 10/21/2009 14:12'!
posters

	"An array pairs (integer , content descriptor).   
	 The integer is a panel index - should fit in the number of #panelLocations.

	 The method name posters predates the QPoster object...
 	 this is template protocol for specifying QDisplayPanel contents, not QPosters."
	
	^ { 
		{4 . {#text . (self initialEditableText) }}.
	}.! !

!QForumTemplate methodsFor: 'content' stamp: 'ar 4/16/2011 17:41'!
rolodexCards
	"Return a array of pairs, each pair a rolodex index followed by content descriptor."
	^  { 
		{ 1 . { #url . 'http://google.com'}}.
		{ 1.  { #url . 'http://www.openqwaq.com' }}.	
	}
! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 3/10/2007 14:34'!
rolodexLocations

	"The ordered list of freestanding rolodex positions;
		{ x@y . height . rotationDegrees }
	"

	| height x z |
	x := (2*self extent x)/5.
	z := (2*self extent z)/5.
	height := 3.8.

	^ { 
		"clockwise from northwest"
		{ (x negated)@(z negated). height . 45 } .
		{ x @ (z negated) . height . -45 } .
		{ x @ z . height . -135 } .
		{ x negated@z . height . 135 } .
	
	}! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 3/9/2007 22:19'!
roomHeight
	^ self extent y! !

!QForumTemplate methodsFor: 'accessing' stamp: 'bgf 3/10/2007 00:15'!
space

	"the QwaqRoomSpace we are creating"
	^ space! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 4/8/2007 11:21'!
surroundingSectionWalls: aTextureFileName

	"Surround walls, where the texture is a repeating vertical strip."

	| |
	
	^ self surroundingWalls: aTextureFileName dX: 2 dY: (self extent y).! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 4/8/2007 11:20'!
surroundingWalls: aTextureFileName

	" Return an array of the 
		{ { x@y . x@y } ... }  endpoints of four regular height walls enclosing the room.
	  ."

	| |
	
	^ self  surroundingWalls: aTextureFileName dX: 2 dY: 2! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 4/8/2007 23:42'!
surroundingWalls: aTextureFileName dX: dx dY: dy

	" Return an array of the 
		{ { x@y . x@y } ... }  endpoints of four regular height walls enclosing the room.

		dX is the x repetition as a divisor of total width of the surface;
	 	dY is the y repetition as a divisor of total height of the surface.
	  ."

	| x z tx ty tz |
	x := self extent x / 2.
	z := self extent z / 2.

	tx := self extent x / dx.
	ty := self extent y / dy - 0.01.
	tz := self extent z / dx.

	^ { 
		{x@z . x@(z negated) . aTextureFileName . tz@ty} .
		{x@z . (x negated)@z . aTextureFileName . tx@ty} .
		{(x negated)@z . (x negated)@(z negated) . aTextureFileName . tz@ty } .
		{x@(z negated) . (x negated)@(z negated) . aTextureFileName . tx@ty} 
	}.! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 3/13/2007 19:13'!
tables
	"Plain tables - each entry a pair, position and then yaw degrees."
	^ { }! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 7/14/2007 16:20'!
vaultColor
	"IF non-nil, a custom color (both ambient and diffuse) for the ceiling vault)"
	^ nil! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/22/2007 01:26'!
wallMaterial
	^ (TMaterial new)	
		diffuseColor: (Color gray: 0.6);
		ambientColor: (Color gray: 0.8);
		emissiveColor: (Color gray: 0.05);
		yourself.! !

!QForumTemplate methodsFor: 'layout' stamp: 'bgf 9/21/2007 22:42'!
walls
	"Wall descriptions."
	| result |
	result := self surroundingSectionWalls: 'WallSectionWood.jpg'.

	"We use a duller/darker basic unit on the two walls that have portals in them;
	 which are also the corner/half defined by the floor paths. 
	 Gives a bit of global orientation for users."

	(result at: 2) at: 3 put: 'WallSectionWoodDark.jpg'.
	(result at: 3) at: 3 put: 'WallSectionWoodDark.jpg'.

	^ result
! !

QForumTemplate subclass: #QGalleryForum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QGalleryForum commentStamp: '<historical>' prior: 0!
A large exhibition-space forum  (200x300 feet)
subdivided with many walls.

Starts without furniture.!

!QGalleryForum class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QGalleryForum class methodsFor: 'as yet unclassified' stamp: 'bgf 3/10/2007 02:07'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Gallery'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QGalleryForum class methodsFor: 'as yet unclassified' stamp: 'bgf 3/15/2007 17:14'!
islandName
	^ 'Gallery'! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 10:10'!
ceilingMaterial

	^ (TMaterial new)
		color: (Color r:0.76 g:0.75 b:0.73);
		texture: (QTextures find: (self ceilingTextureFile));
		yourself
! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 7/14/2007 15:56'!
ceilingUVScale
	^ (self extent x / 2) @ (self extent z / 4)! !

!QGalleryForum methodsFor: 'content' stamp: 'bgf 3/13/2007 21:59'!
entryDescription
	^ nil
	"Don't preset a link, so the entry is found first by automatic-linking."! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 1/26/2008 17:14'!
extent
	^ 200@16@300! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/8/2007 13:25'!
extraPanelLocations

	"The positions and orientations for overflow display panels."
	| result perSide yaw spacing walls |

	"Just plot all possible panel locations."
	spacing := 14. "Center to center"
	result := OrderedCollection new.

	walls := space walls asOrderedCollection.
	walls := walls select: [ :w |
			w material texture fileName endsWith: 'simpleWall.jpg'
		  ].
	walls do: [ : w |
		perSide := (((w extent x -spacing) // spacing) -1) max: 0.
		yaw := w globalTransform pitchYawRoll y.
		(perSide // -2) to: (perSide // 2) do: [ :i |
			result add: {
				w localToGlobal: (i * spacing)@5@0 .
				yaw - 180.0.
			}.
		].
	].

false ifTrue: [	"It is easier to believe the ugprade result if the outward-facing sides
	 			 of the walls are not used - so you can see all the pictures while wandering.
				 The upgrade slot assigner doesn't take direction into account,
	 			 just the bearing from the origin of the room."

	"And then the other sides of the walls (after excluding the exterior walls.)"
	walls := walls reject: [ :w | 
		(w globalPosition x abs > (self extent x /2.0 - 5)) or: [
		(w globalPosition y abs > (self extent y / 2.0 - 5))].
	].

	walls do: [ :w |
		perSide := (((w extent x -spacing) // spacing) -1) max: 0.
		yaw := w globalTransform pitchYawRoll y.
		(perSide // -2) to: (perSide // 2) do: [ :i |
			result add: {
				w localToGlobal: (i * spacing)@5@0 .
				yaw
			}.
		].
	].
].

	^ result! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 3/10/2007 10:34'!
fanLocations
	^ #()! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:38'!
floorMaterial

	"Default material is formed from default texture file, 
	 on a light gray material.  A nil result here suppresses the floor."	

	self floorTextureFile ifNil: [ ^ nil ].

	^  (TMaterial new)
		color: (Color r:0.62 g: 0.60 b: 0.58);
		texture: (QTextures find: (self floorTextureFile))! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 7/15/2007 17:51'!
floorTextureFile
	^ 'carpetTan.jpg'! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:39'!
floorUVScale
	^ self extent x/5.0@(self extent z/8.0)! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/2/2007 17:46'!
gateLocations
 	^ {
		{-28@-150 . 0 . #EntryPortal }.
		{(self extent x / 2 - 20)@150 . 180 }.
		}! !

!QGalleryForum methodsFor: 'layout' stamp: 'jdm 9/26/2007 15:42'!
homeTransform
	"The default landing space when logging / teleporting into the space."
	^ (Matrix4x4 new translation: -30@5.5@-142) rotationAroundY:-160! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 3/10/2007 11:03'!
landmarks
 	^ {
		{'Bird''s Eye' . -110@220@-160 . -53@-145@0 . 'b' }.
		{'Entry' . -5@0@-140 . 0@-160@0 . 'c'}.
		{'East Promenade' . -90@0@-140 . 0@-180@0 . 'd' } .
		{'South Gallery' . -72@0@46 . 0@-120@0 . 'e' } .
		{'West Gallery' . 62@0@120 . 0@5@0 . 'f' }.
		{'North Gallery' . 74@0@-21.7 . 0@50@0. 'g' }.
	}! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 10/4/2007 12:31'!
mirrorLocations
	
	"Near the gate positions."

 	^ {
		{-40@5@-150 . 0 }.
		{(self extent x / 2 - 32)@5@150 . 180 }.
		}! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/4/2008 12:20'!
noteboardLocations
	"The positions and orientations for brainstorming areas"
	^ #()! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 10/4/2007 12:30'!
panelLocations

	"The positions and orientations for initial display panels."
	^ { 
	}.! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 3/10/2007 02:42'!
pedestalLocations
	^ #()! !

!QGalleryForum methodsFor: 'content' stamp: 'bgf 3/10/2007 02:41'!
pedestals
	^ #()! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 10:38'!
placeCustomElements

	super placeCustomElements.
	"Some of the outer walls are partly open to the sky... so make a sky."
	space addChild: ((QSkyDome new) style:#normal; scale: 0.45; translation: 0@10@0; yourself).
	self placeOuterRails.! !

!QGalleryForum methodsFor: 'content' stamp: 'bgf 9/7/2007 09:51'!
placeCustomRugs

	"Lay down some carpets to make the areas distinguishable.
	 Colored strips down the side galleries so they feel different to one another,
	 and a couple of large masses in the main spaces."

	| mat cent ext |
	mat := self floorMaterial.
	mat color: (mat diffuseColor asColor darker mixed: 0.9 with: (Color r:1 g:0.0 b:0.1)).
	cent := (self extent x / 2.0 - 20) @ 0.
	ext := 14@(self extent z - 60).
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'west-red-rug' copy. 

	mat := self floorMaterial.
	mat color: (mat diffuseColor asColor darker mixed: 0.9 with: (Color r:0.05 g:0.7 b:0.5)).
	cent := (self extent x / -2.0 + 20) @ 0.
	ext := 14@(self extent z - 60).
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'east-green-rug' copy. 

	mat := self floorMaterial.
	mat color: (mat diffuseColor asColor mixed: 0.8 with: (Color brown)).
	cent := 0@(self extent z/4).
	ext := 64@36.
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'north-brown-rug' copy. 

	mat := self floorMaterial.
	mat color: (mat diffuseColor asColor darker mixed: 0.85 with: (Color r:0 g:0.3 b:1.0)).
	cent := 0@(self extent z/-4).
	ext := 64@64.
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'south-blue-rug' copy. ! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 10:38'!
placeOuterRails

	"Run low walls outside the 'main' walls, so the empty stretches 
	 don't just fall out into space... and it's harder to see the crummy bottom of the skybox."

	| x z tx ty tz dx |

	dx := 4.0.
	x := self extent x / 2 + 1.
	z := self extent z / 2 + 1.

	tx := self extent x / dx.
	ty := 1.6.
	tz := self extent z / dx.

	 { 
		{x@z . x@(z negated) . tz@ty} .
		{x@z . (x negated)@z . tx@ty} .
		{(x negated)@z . (x negated)@(z negated) . tz@ty } .
		{x@(z negated) . (x negated)@(z negated) . tx@ty} 
	} do: [ :spec|
		space placeWallFrom: spec first to: spec second 
			   height: 3.2
			   material: (self wallMaterial) 
			   texture: (QTextures find: 'brick012.jpg')
			   repeat: spec third.
	].! !

!QGalleryForum methodsFor: 'content' stamp: 'bgf 3/10/2007 02:23'!
posters
	^#()! !

!QGalleryForum methodsFor: 'content' stamp: 'bgf 3/10/2007 02:40'!
rolodexCards
	^ #()! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 3/10/2007 02:43'!
rolodexLocations
	^ #()! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:58'!
vaultColor
	^ VectorColor r:0.8 g: 0.81 b: 0.79! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 7/18/2007 01:05'!
wallMaterial
	^ (TMaterial new)	
		color: (VectorColor r:0.4 g:0.4 b:0.4);
		emissiveColor: (VectorColor r: 0.55 g: 0.55 b:0.55);
		yourself.! !

!QGalleryForum methodsFor: 'layout' stamp: 'bgf 9/28/2007 17:17'!
walls

	" { { x@y . x@y } ... }  endpoints of all the regular-height walls.
	  The default result is a simple foursquare enclosure of the room."

	| paint brick |
	paint := 'simpleWall.jpg'.
	brick := 'brick012.jpg'.

	^ { 
			"List the interior walls before exterior, 
			 since these are the preferred spots when renovating.
			 The endpoints are supposed to be ordered so that the 'front' of the wall
	 		 is inward-facing... this allows good results when running the forum-converter."
			{ -35@40 . 35@-20 . paint . 20@1 }.

			{-60@-120 . -60@40 . paint . 20@1 }.
			{-60@60 . -60@120 . paint . 20@1 }.

			{ 60@-40 . 60@-120 . paint . 20@1 }.
			{ 60@100 . 60@-20. paint . 20@1 }.

			{100@60 . 100@-60 . paint . 20@1 }.
			{100@-100 . 100@-150 . brick . 10@5 }.

			{-100@-150 . -100@-60 . paint. 20@1 }.
			{-100@20 .  -100@140 . paint. 20@1 }.

			{-100@-150 . 100@-150 . brick . 40@5 }.
			{-100@150 .  100@150 . paint . 40@1 } .
	}.! !

QForumTemplate subclass: #QPersonalOffice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QPersonalOffice commentStamp: '<historical>' prior: 0!
A small room with a few display panels,
including a mirror,  and a table just so there's something.
!

QPersonalOffice subclass: #QOfficeModern
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QOfficeModern commentStamp: '<historical>' prior: 0!
An office with high-tone modern styling; 
the walls are faux-shiny orange panels 
with strong gradient lighting effects.
The floors and ceiling are very dark.!

!QOfficeModern class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QOfficeModern class methodsFor: 'service' stamp: 'bgf 9/4/2008 12:53'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Office (Warm)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:37'!
ceilingMaterial

	^(TMaterial new)
		color: (Color r:0.99 g:0.97 b:0.92) ;
		texture: ( QTextures find: self ceilingTextureFile);
		yourself.! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:22'!
ceilingTextureFile
	^  'CeilingSquare.gif' ! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 4/14/2007 22:17'!
ceilingUVScale	

	^ 5@8! !

!QOfficeModern methodsFor: 'content' stamp: 'bgf 4/7/2007 16:40'!
entryDescription
	^ nil
	"Don't preset a link, so the entry is found first by automatic-linking."! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 1/26/2008 17:14'!
extent
	^ 40@12@60! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 9/22/2007 00:37'!
floorMaterial

	^ (TMaterial new) 
		color: (Color r:0.44 g:0.40 b:0.36);
		texture: (QTextures find:(self floorTextureFile));
		yourself.
! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 4/7/2007 21:50'!
floorTextureFile
	^ 'greyFlannel.jpg'! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 7/14/2007 18:11'!
gateLocations
 	^ {
		{0@-30 . 0 . #EntryPortal }.
		{-20@0 . 90  }.
	}! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:38'!
vaultColor
	^ VectorColor r:0.75 g: 0.73 b: 0.67! !

!QOfficeModern methodsFor: 'layout' stamp: 'bgf 9/22/2007 00:30'!
walls

	"Wall descriptions."
	| result |
	result :=  self surroundingSectionWalls: 'WallSectionWarm.gif'.

	"We use a duller/darker basic unit on the two walls that have portals in them;
	 which are also the corner/half defined by the floor paths. 
	 Gives a bit of global orientation for users."

	(result at: 3) at: 3 put: 'WallSectionWarmDark.gif'.
	(result at: 4) at: 3 put: 'WallSectionWarmDark.gif'.

	^ result! !

QOfficeModern subclass: #QOfficeModernCool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QOfficeModernCool commentStamp: '<historical>' prior: 0!
A variant modern-style office with cooler-hued blue-steel walls.!

!QOfficeModernCool class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QOfficeModernCool class methodsFor: 'service' stamp: 'bgf 9/4/2008 12:53'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Office (Cool)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QOfficeModernCool methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:41'!
ceilingMaterial

	^(TMaterial new)
		color: (Color r:0.93 g:0.96 b:0.99) ;
		texture: ( QTextures find: self ceilingTextureFile);
		yourself.! !

!QOfficeModernCool methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:31'!
ceilingTextureFile
	^  'CeilingSquare.gif' ! !

!QOfficeModernCool methodsFor: 'layout' stamp: 'bgf 4/18/2007 11:16'!
floorMaterial

	^ (TMaterial new) 
		color: (Color r:0.40 g:0.42 b:0.43);
		texture: (QTextures find:(self floorTextureFile));
		yourself.
! !

!QOfficeModernCool methodsFor: 'layout' stamp: 'bgf 9/23/2007 09:41'!
vaultColor
	^ VectorColor r:0.72 g: 0.74 b: 0.77! !

!QOfficeModernCool methodsFor: 'layout' stamp: 'bgf 9/22/2007 00:55'!
walls
	"Wall descriptions."
	| result |
	result :=  self surroundingSectionWalls: 'WallSectionCoolDark.gif'.

	"Highlite walls with more color."

	(result at: 1) at: 3 put: 'WallSectionCool.gif'.
	(result at: 2) at: 3 put: 'WallSectionCool.gif'.

	^ result! !

!QPersonalOffice class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QPersonalOffice class methodsFor: 'service' stamp: 'bgf 9/4/2008 12:53'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Office (Light)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QPersonalOffice class methodsFor: 'service' stamp: 'bgf 3/15/2007 17:13'!
islandName
	^ 'Office'! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 7/14/2007 18:17'!
ceilingMaterial

	^(TMaterial new)
		color: (Color r:0.96 g:0.955 b:0.94) ;
		texture: ( QTextures find: self ceilingTextureFile);
		yourself.! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 4/8/2007 15:01'!
ceilingTextureFile
	^ 'ceiling.jpg' ! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 7/14/2007 17:39'!
ceilingUVScale
	^ 2@1! !

!QPersonalOffice methodsFor: 'content' stamp: 'bgf 3/13/2007 19:48'!
entryDescription
	^ nil
	"Don't preset a link, so the entry is found first by automatic-linking."! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 1/26/2008 17:13'!
extent
	^ 32@11.5@56! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 9/6/2007 14:12'!
extraGateLocations

	"Gate locations when extra gates are needed when renovating."
	^ #()! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 9/6/2007 13:58'!
extraPanelLocations

	"The positions and orientations for surplus display panels."

	| height width depth |

	height := 5. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"north wall"
		{ width/4 @ height @ (depth/-2) . 0 } .
		{ width/-4 @ height @ (depth/-2) . 0 } .
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 3/16/2007 16:54'!
fanLocations
	^ #()! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:29'!
floorTextureFile
	 ^ 'tiles002.jpg' ! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:13'!
floorUVScale
	^ self extent x/5.0@(self extent z/10.0) ! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 7/14/2007 18:12'!
gateLocations
 	^ {
		{0@-28 . 0 . #EntryPortal }.
		{-16@0 . 90  }.
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'jdm 8/21/2007 08:17'!
homeTransform
	"The default landing space when logging / teleporting into the space."
	^ (Matrix4x4 new translation: 0@5@-24) rotationAroundY:-180! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 6/21/2008 19:27'!
landmarks
 	^ {
		{'Bird''s Eye' . -20@40@-30 . -50@-145@0 . 'b' }.
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 10/4/2007 14:49'!
mirrorLocations

	"The positions and orientations for initial display panels."

	| height width depth |

	height := 5. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"south wall" 
		{ 0 @ height @ (depth/2.0) . 180 } .
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 9/4/2008 12:20'!
noteboardLocations

	"The positions and orientations for initial display panels."

	| height width depth |

	height := 5.2. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"west wall" 
		{ width/-2.0 @ height @ (depth/4.2) . 90 . 11@7 }
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 9/4/2008 12:01'!
panelLocations

	"The positions and orientations for initial display panels."

	| height width depth |

	height := 5. "= QPlainAvatar's height above ground."
	width := self extent x.
	depth := self extent z.

	^ { 
		"north wall"
		
		"east wall"

		{ width/2.0 @ height @ (depth/-4.5) . -90 } .
		{ width/2.0 @ height @ (depth/4.5) . -90 } .

		"west wall" 
		 { width/-2.0 @ height @ (depth/-4.5) . 90 } .
		" --- { width/-2.0 @ height @ (depth/4.5) . 90 } .   -- brainstorm "
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 3/13/2007 18:53'!
pedestalLocations
	^ #()! !

!QPersonalOffice methodsFor: 'content' stamp: 'bgf 3/13/2007 18:53'!
pedestals
	^ #()! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 9/7/2007 00:47'!
placeCustomRugs

	"Lay down some carpets to make the areas distinguishable.
	 A couple of doormats, and a bigger carpet under the desk."

	| baseColor mat cent ext |
	baseColor := space floor material diffuseColor asColor darker.
	mat := QForumTemplate sisalCarpet.
	mat color: (baseColor mixed: 0.85 with: (Color r:0.8 g:0.2 b:0.1)).
	cent := (self extent x / -2.0 + 3) @ 0.
	ext :=6@9.
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'red-rug' copy. 

	mat := QForumTemplate sisalCarpet.
	mat color: (baseColor mixed: 0.8 with: (Color r:0.15 g:0.25 b:0.55)).
	cent := 0@(self extent z / -2.0 + 3).
	ext := 9@6.
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'green-rug' copy. 

	mat := QForumTemplate sisalCarpet.
	mat color: (baseColor).
	ext := ((self extent x / 2)@(self extent z /2 ))+ 6.
	cent := (ext/2) - 6.
	(space floor addRugAt: cent extent: ext material: mat)
		uvScale: ext/3;
		objectName: 'desk-rug' copy. 
	
	! !

!QPersonalOffice methodsFor: 'content' stamp: 'bgf 10/4/2007 11:22'!
posters
	^{
	  }! !

!QPersonalOffice methodsFor: 'content' stamp: 'bgf 3/13/2007 18:53'!
rolodexCards
	^ #()! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 7/14/2007 18:13'!
rolodexLocations
	^ { {-14@25 . 3.5 . 135} }! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 4/13/2008 10:39'!
tables
	"A desk"
	^ { 
		{5@0@9 . 30 . 9@2.6@4.5}
	}! !

!QPersonalOffice methodsFor: 'layout' stamp: 'bgf 4/8/2007 23:03'!
walls
	^ self surroundingSectionWalls: 'WallSectionOffice.gif'! !

QForumTemplate subclass: #QProjectForum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QProjectForum commentStamp: '<historical>' prior: 0!
The prototypical project war-room style forum.!

QProjectForum subclass: #QForumModernBlue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QForumModernBlue commentStamp: '<historical>' prior: 0!
A project forum with cool blue walls.!

!QForumModernBlue class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumModernBlue class methodsFor: 'islands' stamp: 'bgf 4/8/2007 15:08'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Conference (Blue)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QForumModernBlue methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:22'!
ceilingMaterial

	"Default material is formed from default texture file, 
	 on a light gray material.  A nil result here suppresses the ceiling."	

	self ceilingTextureFile ifNil: [ ^ nil ].

	^  (TMaterial new)
		color: (Color r:0.9 g:0.9 b:0.91);
		texture: (QTextures find: (self ceilingTextureFile))
! !

!QForumModernBlue methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:30'!
floorMaterial

	^  (TMaterial new)
		color: (Color r:0.66 g: 0.635 b: 0.62);
		texture: (QTextures find: (self floorTextureFile))! !

!QForumModernBlue methodsFor: 'layout' stamp: 'bgf 5/1/2007 15:02'!
walls

	"Wall descriptions."
	| result |
	result := self surroundingSectionWalls: 'WallSectionBlue.gif'.
	(result at: 2) at: 3 put: 'WallSectionBlueDark.gif'.
	(result at: 3) at: 3 put: 'WallSectionBlueDark.gif'.

	^ result! !

QProjectForum subclass: #QForumModernRose
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QForumModernRose commentStamp: '<historical>' prior: 0!
A modern-style conference room with a smoky pink wall tone.!

!QForumModernRose class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumModernRose class methodsFor: 'islands' stamp: 'bgf 4/8/2007 15:09'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Conference (Rose)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QForumModernRose methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:28'!
ceilingMaterial

	"Default material is formed from default texture file, 
	 on a light gray material.  A nil result here suppresses the ceiling."	

	self ceilingTextureFile ifNil: [ ^ nil ].

	^  (TMaterial new)
		color: (Color r:0.94 g:0.91 b:0.92);
		texture: (QTextures find: (self ceilingTextureFile))
! !

!QForumModernRose methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:29'!
floorMaterial

	^  (TMaterial new)
		color: (Color r:0.68 g: 0.62 b: 0.59);
		texture: (QTextures find: (self floorTextureFile))! !

!QForumModernRose methodsFor: 'layout' stamp: 'bgf 5/1/2007 15:03'!
walls

	"Wall descriptions."
	| result |
	result := self surroundingSectionWalls: 'WallSectionRose.gif'.
	(result at: 2) at: 3 put: 'WallSectionRoseDark.gif'.
	(result at: 3) at: 3 put: 'WallSectionRoseDark.gif'.

	^ result! !

QProjectForum subclass: #QJasmineForum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QJasmineForum commentStamp: '<historical>' prior: 0!
An entry forum for the Jasmine+Forum demo.
A plain project forum, with links to the Jasmine Intro World 
for the entry portal and a pedestal.

See the class side, where the service description
includes all the islands of the Jasmine BFD spaces.!

!QJasmineForum class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QJasmineForum class methodsFor: 'islands' stamp: 'ar 7/7/2007 16:04'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Jasmine Demo Forum'
			entryWorld: self 
			extraWorlds: (#(
				IntroWorld 
				T3BodyWorld 
				DungeonWorld 
				MarsWorld
				UnderwaterWorld
				SpreadSheetWorld
			) collect:[:worldName| Smalltalk classNamed: worldName] thenSelect:[:world| world notNil]) 
			clientUIClass: QJasmineForumDemo! !

!QJasmineForum class methodsFor: 'islands' stamp: 'bgf 4/27/2007 21:45'!
islandName
	^ 'Jasmine Forum'! !

!QJasmineForum methodsFor: 'content' stamp: 'bgf 3/15/2007 10:05'!
entryDescription
	"A list {'label', url, isOpen} defining the north portal's content and label;
	 the last item is boolean; if true, the north portal starts out open. "

	| |
	^ { 'Jasmine' . '../Intro World/QwaqPortal' . false}! !

!QJasmineForum methodsFor: 'content' stamp: 'bgf 4/27/2007 21:47'!
pedestals
	^ {
			{'Intro World' . #mainEntry . 0.09 . 0@5@0  .'Jasmine Demo'}.
			{self class islandName . #mainEntry . 0.05 . 0@28@0 . 'This Forum'}
		}! !

QProjectForum subclass: #QMinimalForum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QMinimalForum commentStamp: '<historical>' prior: 0!
The conference room 'Sample Forum' template.

This is called 'QMinimalForum' for historical reasons;
old persisted demos refer to this class by name.

A default-sized conference room with a bit of default content.
!

!QMinimalForum class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QMinimalForum class methodsFor: 'islands' stamp: 'ar 4/16/2011 17:13'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Sample Forum'
			entryWorld: self 
			clientUIClass: QwaqForumsUI
			! !

!QMinimalForum class methodsFor: 'islands' stamp: 'ar 4/16/2011 17:13'!
islandName
	^ 'Sample Forum'! !

!QMinimalForum methodsFor: 'layout' stamp: 'bgf 12/21/2007 12:03'!
demoDesksUnused

	"Peering desk positions:
		 each is  {location . yaw } ...} 
	"

	| radius adjust base n result |

	n := 2.
	radius := (self extent x min: self extent z)/2.4.
	adjust := n even ifTrue: [n/2] ifFalse: [(n+1)/2].
	base := n even ifTrue: [36] ifFalse: [21].

	result := OrderedCollection new.		
	1 to: n do: [ :i|
		| yaw |
		yaw := (i-adjust)*30 - base.
		result addLast: { 
			((yaw degreeSin*radius)-14)@0@(((yaw degreeCos) *radius) -10) 
			. yaw }
			"x offset moves the arc center a bit to the left, to free up right-hand space
			 around the text editor for the demo.  z offset moves the arc to the rear of the room."
	].

	^result! !

!QMinimalForum methodsFor: 'initialize' stamp: 'bgf 3/16/2007 12:30'!
entryDescription
	"A list {'label', url, isOpen} defining the north portal's content and label;
	 the last item is boolean; if true, the north portal starts out open. "

	| |
	^nil! !

!QMinimalForum methodsFor: 'content' stamp: 'ar 4/16/2011 17:13'!
pedestals

	^ {
		{self class islandName . #mainEntry . 0.05 . 0@28@0 . 'This Forum' }.
	}.! !

!QMinimalForum methodsFor: 'content' stamp: 'bgf 10/21/2009 14:12'!
posters
		^ { 		
			{3 . {#texture . 'orgChart.jpg'}}.
			{4 . {#text . (self initialEditableText) }}.
			{5 . {#texture . 'oilRatioMD.gif'}}.
	    		{6 . {#texture . 'sat300mLg.gif'}} .
		}! !

!QMinimalForum methodsFor: 'content' stamp: 'ar 4/16/2011 17:43'!
rolodexCards

	^ { 
		{ 1 . { #url . 'http://google.com'}}.
		{ 1.  { #url . 'http://www.openqwaq.com' }}.
		
	      { 2 . { #texture . 'orgChart.jpg'}}.

		{ 3 . { #texture .'sat300mLg.gif'}}.
		{ 3 . { #texture . 'PERT.gif'}}.
		{ 3 . { #texture . 'oilRatioMD.gif'}}.

	  }. ! !

!QMinimalForum methodsFor: 'layout' stamp: 'bgf 4/13/2008 10:32'!
tables
	"one table - in the corner between the portals"
	^ { { -23@0@17 . 0 . 8@2.7@4 }}! !

QProjectForum subclass: #QMinimalForumNaCl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums'!

!QMinimalForumNaCl class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QMinimalForumNaCl class methodsFor: 'islands' stamp: 'das 2/27/2007 20:38'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'NaCl Forum'
			entryWorld: self 
			clientUIClass: QwaqForumsUI
			! !

!QMinimalForumNaCl class methodsFor: 'islands' stamp: 'das 2/27/2007 20:38'!
islandID
	^TObjectID readHexFrom: 'd3c71aa7656049086c633dec87594c09'

! !

!QMinimalForumNaCl class methodsFor: 'islands' stamp: 'das 2/27/2007 20:38'!
islandName
	^ 'Qwaq NaCl Forum'! !

!QMinimalForumNaCl methodsFor: 'initialize' stamp: 'bgf 3/10/2007 14:06'!
entryDescription
	"A list {'label', url, isOpen} defining the north portal's content and label;
	 the last item is boolean; if true, the north portal starts out open. "

	| |

	^ { 'NaCl' . '../', self class islandName, '/NaClForumGate' . false}! !

!QMinimalForumNaCl methodsFor: 'initialize' stamp: 'bgf 5/31/2009 19:51'!
initialize

	| park |
	super initialize.

	park := QForumNaCl new.
	park populate.
	(park addGate: #NaClForumGate label: 'Forum')
		postcardLink: (space entryPortal postcard).

	park markAsTemplateInstance.! !

!QMinimalForumNaCl methodsFor: 'initialize' stamp: 'bgf 9/2/2007 10:40'!
pedestals

	^ {
		{self class islandName . #NaCl. 0.31 . 4.586@4.2@20.22 . 'NaCl'.} .
		{self class islandName . #mainEntry . 0.05 . 0@28@0 . 'Conference Room'}
	}.! !

!QProjectForum class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QProjectForum class methodsFor: 'islands' stamp: 'bgf 9/4/2008 12:22'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Conference (Birch)'
			entryWorld: self 
			clientUIClass: QwaqForumsUI
			! !

!QProjectForum class methodsFor: 'islands' stamp: 'ar 4/16/2011 17:14'!
islandName
	^ 'Forum'! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 9/7/2007 09:21'!
ceilingMaterial

	^  (TMaterial new)
		color: (Color r:0.95 g:0.92 b:0.9);
		texture: (QTextures find: (self ceilingTextureFile))
! !

!QProjectForum methodsFor: 'initialize' stamp: 'bgf 3/15/2007 16:47'!
entryDescription
	"A list {'label', url, isOpen} defining the north portal's content and label;
	 the last item is boolean; if true, the north portal starts out open. "

	| |
	"A generic conference rooms start life unconnected."

	^ nil! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 9/6/2007 15:10'!
extraGateLocations

	"Gate locations when extra gates are needed when renovating."
	^ { 
		{ 8.5@(self extent z/2) . 180} .
		{ -8.5@(self extent z/2) . 180}.
	}! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 9/22/2007 01:00'!
floorMaterial

	^  (TMaterial new)
		color: (Color r:0.65 g: 0.61 b: 0.56);
		texture: (QTextures find: (self floorTextureFile))! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 5/1/2007 14:07'!
floorPaths
	^ self floorPathsToGates! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 4/8/2007 02:51'!
floorTextureFile
	^ 'carpetTan.jpg'! !

!QProjectForum methodsFor: 'content' stamp: 'bgf 3/15/2007 16:46'!
pedestals

	^ {
		 nil .
		{self class islandName . #mainEntry . 0.05 . 0@28@0 . 'This Forum'}
	}.! !

!QProjectForum methodsFor: 'extras' stamp: 'bgf 9/4/2007 14:09'!
placeCustomRugs

	"QFloor knows how to layout color-coded quadrant rugs..."
	space floor addQuarterRugs.! !

!QProjectForum methodsFor: 'content' stamp: 'bgf 10/21/2009 14:12'!
posters

	^ { 
			{4 . {#text . (self initialEditableText) }}.
		}.! !

!QProjectForum methodsFor: 'content' stamp: 'ar 4/16/2011 17:43'!
rolodexCards
	^  { " room 1"
		{ 1 . { #url . 'http://google.com'}}.
		{ 1.  { #url . 'http://www.openqwaq.com' }}.
	}
! !

!QProjectForum methodsFor: 'layout' stamp: 'bgf 4/13/2008 10:39'!
tables

	"one table - in the corner between the portals"
	^ { { -23@0@17 . 0 . 8.2@2.7@4 }}.! !

QForumTemplate subclass: #QStagingArea
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Templates'!

!QStagingArea commentStamp: '<historical>' prior: 0!
A large empty area; no walls, ceiling, or content.!

!QStagingArea class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QStagingArea class methodsFor: 'instance creation' stamp: 'bgf 3/22/2007 21:04'!
descriptionForCroquetService

	^ QServiceDescription 
			name: 'Staging Area'
			entryWorld: self 
			clientUIClass: QwaqForumsUI! !

!QStagingArea class methodsFor: 'defaults' stamp: 'bgf 5/15/2007 12:30'!
islandName
	^ 'Space'	"Generic, so it is reusable."! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:02'!
ceilingTextureFile
	^ nil! !

!QStagingArea methodsFor: 'content' stamp: 'bgf 3/22/2007 20:55'!
entryDescription
	^ nil! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:06'!
extent
	^ 500@100@500! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 9/6/2007 14:11'!
extraGateLocations
	^ #()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 9/6/2007 15:24'!
extraPanelLocations

	| result |
	result := OrderedCollection new.
	1 to: 10 do: [ :i |
		result add: { (i*12)-6 @ 5 @ -15 . 0 }.
		result add: { (i*-12)+6 @ 5 @ -15  . 0 } .
	].
	^ result.! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 20:56'!
fanLocations
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 5/1/2007 13:28'!
floorPathMaterial
	^ nil! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:29'!
floorTextureFile
	^ 'ground002.jpg'
	! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 4/7/2007 17:13'!
floorUVScale
	^ 50@50
	! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 20:58'!
gateLocations

	^ { { 0@(self extent z/2) . 180 . #EntryPortal} } .! !

!QStagingArea methodsFor: 'layout' stamp: 'jdm 8/21/2007 08:17'!
homeTransform
	^ Matrix4x4 new translation: 0@5@0! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 6/21/2008 19:16'!
landmarks
	^ {
		{'Bird''s Eye' . 250@125@250 . -30@45@0 . 'b' } .
		{'Vista' . 250@30@250 . -5@45@0 . 'v' }
		}! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 10/4/2007 11:20'!
mirrorLocations
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 9/4/2008 12:20'!
noteboardLocations
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:02'!
panelLocations
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:02'!
pedestalLocations
	^#()! !

!QStagingArea methodsFor: 'content' stamp: 'bgf 3/22/2007 20:55'!
pedestals
	^ #()! !

!QStagingArea methodsFor: 'as yet unclassified' stamp: 'bgf 9/4/2008 12:52'!
placeCustomElements

	super placeCustomElements.
	"Big empty space - needs a sky to feel alive."
	space addChild: ((QSkyDome new) style:#normal; scale: 1.2; translation: 0@-10@0; yourself).! !

!QStagingArea methodsFor: 'content' stamp: 'bgf 3/22/2007 20:55'!
posters
	^ #()! !

!QStagingArea methodsFor: 'content' stamp: 'bgf 3/22/2007 20:55'!
rolodexCards
	^ #()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:02'!
rolodexLocations
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:03'!
roomHeight
	^ 50! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:03'!
tables
	^#()! !

!QStagingArea methodsFor: 'layout' stamp: 'bgf 3/22/2007 21:03'!
walls
	^#()! !

QSimpleToy subclass: #Butterfly
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!Butterfly commentStamp: '<historical>' prior: 0!
Each instance is a fluttering butterfly.

Should be QButterfly... but it is too late to change the name;
there are many extant butterflies in the field.!

!Butterfly class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!Butterfly methodsFor: 'initialize' stamp: 'bgf 3/28/2009 22:06'!
initialize
	|  msh |
	super initialize.
	self isGroundBound: false.
	self isAttractedToSurfaces: false.
	"Until further notice."

	"This loads the frame-thumb butterfly; the real butterfly is a resource."
	msh := TSceneReader new loadFileNamed: 'Objects/Qwaq/butterfly.c3x'.
	self mesh: msh.
	self gripForMoving: msh
! !

!Butterfly methodsFor: 'initialize' stamp: 'bgf 2/3/2007 18:05'!
scaleAsToy
	^ 0.9! !

!Butterfly methodsFor: 'step' stamp: 'bgf 2/12/2010 14:16'!
singleStep
	"Flutter.  I'm a butterfly.  It's what I do."
	| v |
	v := self lookSide.
	v := (v cross:(0@1@0))*0.04* (self scale) negated.
	self translation: (self translation + v + (0@((-2 to:2)atRandom * (self scale) /(30.0))@0)).
	self addRotationAroundY: (-2 to: 5) atRandom / ((self scale)*10.0).   
											"decrease curvature = increase orbit."
	self addRotationAroundX: (-5 to: 5) atRandom / (20.0).  
												

! !

QExteriorSpace subclass: #QForumGarden
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QForumGarden commentStamp: '<historical>' prior: 0!
The simple outdoor space representing a campus;
starts empty, until forums are created and linked.!

!QForumGarden class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 4/3/2009 09:32'!
actionVerbs

	^ super actionVerbs,  {{#resetLights . 'Reset Lighting'}}! !

!QForumGarden methodsFor: 'persist' stamp: 'bgf 11/9/2007 12:25'!
constructFrom: pStream

	super constructFrom: pStream.

	"Argh... work around a problem from version 1.1.9 or so up to 1.1.24...
	 radius was being clobbered by inadvertent use as a temporary."
	(radius = 50) ifTrue: [ radius := self class defaultRadius ].! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 9/21/2007 15:59'!
existingFreeGate

	| |
	^ (self findInstancesOf: QGate) 
		detect: [: gate | gate postcardLink isNil ]
		ifNone: [ nil ].
	! !

!QForumGarden methodsFor: 'initialize' stamp: 'jdm 12/4/2007 20:41'!
floorImage
	^'lawn002.jpg'! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 10/14/2008 15:28'!
freeGatePositions

	| spots gates result |
	spots := self gatePositions.
	gates := (self findInstancesOf: QGate).
	
	result := OrderedCollection new.
	spots do: [ : inSpot |
		| spot |
		spot := inSpot.
		gates do: [ : gate | 
			(spot notNil and: [(gate translation - spot) squaredLength < 25]) ifTrue: [
				spot := nil.
			].
		].
		spot ifNotNil: [ result add: spot ].
	].

	^ result
				! !

!QForumGarden methodsFor: 'accessing' stamp: 'bgf 9/21/2007 16:19'!
gateForNewLink

	| gate spot |

	self existingFreeGate ifNotNilDo: [ :free | ^ free ].
	
	spot := self freeGatePositions 
				ifEmpty: [ 
					self adjustToGround: ((-100 to: 100) atRandom @0 @(-100 to: 100) atRandom)
				] ifNotEmptyDo: [ :free |
					free first.
				].

	gate := (QGate new)
		translation: spot;
		rotationAroundY:  ((spot z abs < Float epsilon) 
							ifTrue: [ 90 * spot x sign ] 
							ifFalse: [ (spot x / spot z) arcTan radiansToDegrees ]);
		yourself.

	self addChild: gate.
	^ gate! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 11/9/2007 11:59'!
gatePositions

	"The predefined gate positions form a gentle semicircle about the origin."
	| rad result spot |
	rad := 50.
	result := OrderedCollection new.
	7.5 to: 90 by: 15 do: [ :theta |
		spot := ((rad * (theta degreeSin))@ (rad * ( theta degreeCos)))  negated.
		result add: (self adjustToGround: spot).
		spot := spot x negated@(spot y).
		result add: (self adjustToGround: spot).
	].

	^ result! !

!QForumGarden methodsFor: 'initialize' stamp: 'bgf 2/4/2010 20:59'!
generateFlatTerrain

	"Generate a flat landscape as my terrain; destroy any previous terrain. "
	| ground terrain |

	terrain := QFixedSurface new.
	terrain objectName: 'Ground'.
	ground := TRectangle new.
	ground
		    objectName: 'Grass';
		    extent: (radius@radius)*2;
		    rotationAroundX: -90;
		    material: (QMaterials groundMaterial: (self floorImage));
		    uvScale: (radius@radius)/6;
		    divisions: (radius / 50) rounded.

	terrain addChild: ground.
	terrain isFromTemplate: true; 
		     doContextMenu: false.
	ground isFromTemplate: true;
		     doContextMenu: false.

	self addChild: terrain.
	^ terrain.

! !

!QForumGarden methodsFor: 'initialize' stamp: 'bgf 2/4/2010 20:59'!
generateFractalTerrain

	"Generate a fractal landscape as my terrain; destroy any previous terrain. "
	| size t terrain |

	size := 2 raisedTo: 4.


	terrain := QCampusGround new.
	1 to: 2 do: [ :i|
		t := QFractalTerrain 
			initializeSize: size 
			smoothness: (1.0 - (0.228*i))
			heightScale: (0.09 - (0.00161 *i)) 
			randomSeed: (Random new seed).
		t genGrid;
			buildMesh;
			boundsDepth: 2;
			 scale: radius;
			uvFactor: (radius/9.0)@(radius/12.0).

		t material: (QMaterials 
			groundMaterial: (#('lawn002.jpg' 'lawn003.jpg' 'ground002.jpg') at: i)).
		" Thiis does very weird things due to z-buffer limitations...
		  try to soften the transitions by making each plane slightly transparent.
		t material diffuseColor:
			(t material diffuseColor copy alpha:(1.0 - (0.01 *(4-i)))).
		t frameChildren do: [ :m | m checkAlpha ].
		"
		terrain addChild: t.	

	].
	self addChild: terrain.
	terrain allUpdateShape.
	^ terrain.

! !

!QForumGarden methodsFor: 'initialize' stamp: 'bgf 3/7/2007 14:56'!
initialize
	super initialize.
	self registerGlobal: #CampusGarden.
! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 8/1/2008 09:08'!
landmarksFor: anAvatar

	"Landmarks available for this avatar in the landmarks menu."
	| places |

	"Build the list automatically in order,
		Home	    (always present)
		Birds Eye  (always present) (maybe my coded one)
		-
		Marked places, gates, apps, etc (inherited)
	"

	places := super landmarksFor:anAvatar.
	places ifNotEmpty: [
		"If the user has added a Bird's Eye, don't fill in my default one."
		(places anySatisfy: [ : p | 'Bird''s Eye' equals: (p label ifNil: ['']) caseSensitive: false])
		 ifTrue: [
				^ places
		].
		^ {places first}, {self birdsEyeLandmark}, (places allButFirst) 
	] ifEmpty: [
		^ {self birdsEyeLandmark}
	].
! !

!QForumGarden methodsFor: 'persist' stamp: 'bgf 1/27/2007 17:12'!
persistOn: pStream

	super persistOn: pStream.
! !

!QForumGarden methodsFor: 'initialize' stamp: 'bgf 10/22/2008 22:49'!
populate
	| tree file |
	super populate.

	file := 'Objects/Qwaq/Plants/TreeHighVetka.c3x'.

	self sky translationX:0 y:40 z:0;
			scale: 1.6.	"So it's out near the edge of a common plan view."

	tree := (QOutdoorTree new)
			setupASE: file aseScale: 0.14 scale:1.0 optimized:true optimizedEarly:true;
			yourself.
	tree translation: (self adjustToGround: (-20@-0.3@30)).
	self addChild:tree.
	tree treeBase translationX: -3.5 y:0 z:-1.0.

	tree :=  (QOutdoorTree new)
			setupASE: file aseScale: 0.09 scale:1.0 optimized:true optimizedEarly:true;
			yourself.
	tree translation: (self adjustToGround: (40@-0.1@60)).
	self addChild:tree.
	tree baseOffset: -3.5@-1.0.

	tree :=  (QOutdoorTree new) 
			setupASE: file  aseScale: 0.16 scale:1.0 optimized:true optimizedEarly:true;
			yourself.
	tree translation: (self adjustToGround: (12@-0.1@-65)).
	self addChild:tree.
	tree baseOffset: -3.5@-1.0.! !

!QForumGarden methodsFor: 'menus' stamp: 'bgf 11/9/2007 11:51'!
resetLights

	"An action verb.  Scour the lights from the space and replace them with defaults."

	(self findInstancesOf: TLight) do: [ : light | light future destroy ].
	lights := Dictionary new.
	self placeLights: (radius * 1.1).
	self daytime.! !

!QForumGarden methodsFor: 'initialize' stamp: 'hrs 3/14/2009 10:59'!
spawnAutoSaveJob
	"Spawn the auto-save loop for this space if one is requested"
	
	^ QAutoSaveJob defaultForForumSpace: self! !

TPointCloud subclass: #QSaltCrystal
	instanceVariableNames: 'length spacing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Widgets'!

!QSaltCrystal commentStamp: '<historical>' prior: 0!
A "procedural" salt crystal
that persists compactly!

!QSaltCrystal class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QSaltCrystal methodsFor: 'persist' stamp: 'bgf 9/2/2007 11:03'!
constructFrom: pStream

	super constructFrom: pStream.
	self length: (pStream readNumber: 'length' default: length).
	self spacing: (pStream readNumber: 'spacing' default: spacing).! !

!QSaltCrystal methodsFor: 'initialize' stamp: 'bgf 11/7/2009 21:20'!
frameBox
	| halfSide |
	halfSide := (length * spacing ).
	^ TBox min: ((halfSide@halfSide@halfSide) - (spacing*0.5)) negated
	             max: ((halfSide@halfSide@halfSide) + (spacing *0.5)).
! !

!QSaltCrystal methodsFor: 'render' stamp: 'bgf 9/2/2007 11:52'!
hasAlpha
	^ false! !

!QSaltCrystal methodsFor: 'render' stamp: 'bgf 9/2/2007 11:52'!
hasOpaque
	^true! !

!QSaltCrystal methodsFor: 'initialize' stamp: 'bgf 9/2/2007 12:06'!
initialize
	super initialize.
	pointSize := 1.
	length := 10.
	spacing := 0.2.
! !

!QSaltCrystal methodsFor: 'accessing' stamp: 'bgf 9/2/2007 11:02'!
length
	"Number of points per side."
	^ length! !

!QSaltCrystal methodsFor: 'accessing' stamp: 'bgf 9/2/2007 11:04'!
length: l
	l ~= length ifTrue: [
		length := l.
		self shapeChanged.
	].! !

!QSaltCrystal methodsFor: 'persist' stamp: 'bgf 9/2/2007 11:04'!
persistOn: pStream
	super persistOn: pStream.
	pStream storeNumber: 'length' value: length.
	pStream storeNumber: 'spacing' value: spacing.! !

!QSaltCrystal methodsFor: 'render' stamp: 'bgf 10/6/2007 12:01'!
pick: pointer

	| side |
	side := (length-1)*2*spacing.
	^ (pointer pickBox: side@side@side location: 0@0@0).! !

!QSaltCrystal methodsFor: 'accessing' stamp: 'bgf 2/11/2010 15:58'!
privateScale: aScalar
	super privateScale: aScalar.
	spacing := spacing * aScalar.
! !

!QSaltCrystal methodsFor: 'transform' stamp: 'bgf 3/17/2008 01:13'!
regenerate
	"Generate the point cloud using current length and scale."

	| len2 len na v c count |
		
	len2 := length//2 +(length\\2) *2.
	len := len2*2.
	v := Vector3Array new:(len*len*len).
	c := VectorColor4Array new:(len*len*len).
	
	na := true.
	count := 1.
	1 to:len do:[:i |
		1 to: len do:[:j |
			1 to: len do:[:k |
				v at:count  put:(Vector3 x: i y: j z: k)-len2* spacing.
				c at: count put:(na ifTrue:[ VectorColor r: 1 g: 0 b: 0 a: 0] ifFalse:[ VectorColor r: 0 g: 0 b: 1 a: 1]).
				count := count + 1.
				na := na not.
			].
			na := na not.
		].
		na := na not.
	].

	self vertices: v.
	self colors: c.! !

!QSaltCrystal methodsFor: 'accessing' stamp: 'bgf 9/2/2007 11:02'!
spacing
	^ spacing! !

!QSaltCrystal methodsFor: 'accessing' stamp: 'bgf 9/2/2007 11:05'!
spacing: aScalar
	spacing ~= aScalar ifTrue: [
		spacing := aScalar.
		self shapeChanged.
	].! !

!QSaltCrystal methodsFor: 'initialize' stamp: 'bgf 11/7/2009 23:22'!
updateBounds
	| box |
	"TFrame part."
	frameChildren ifNotNil:[
		frameChildren do:[:fc | fc updateBounds].].
	
	"Skipping point cloud part.. we don't need to iterate the bounds, we know them."
	box := self frameBox.
	boundSphere := TBoundSphere new.
	boundSphere localPosition: box center.
	boundSphere radius: box radius.
	boundSphere frame: self.
! !

!QSaltCrystal methodsFor: 'initialize' stamp: 'bgf 9/2/2007 10:56'!
updateShape

	"First, generate the point cloud."
	self regenerate.
	super updateShape.
	! !

QFixedSurface subclass: #QCampusGround
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QwaqDemo-Forums-Campus'!

!QCampusGround commentStamp: '<historical>' prior: 0!
Ground object for forum garden; allows dropping links
to create portals linked the mentioned spaces.

Minimal currently; the intention is to wire more sophisticated job-handling
to allow richer automation of forum creation and forum-campus linking.!

!QCampusGround class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QCampusGround class methodsFor: 'defaults' stamp: 'bgf 9/21/2008 16:00'!
defaultDoContextMenu
	"Fewer unintended menus this way.  (Means you need the inspector for editing.)"
	^ false! !

!QCampusGround class methodsFor: 'defaults' stamp: 'bgf 6/4/2009 16:11'!
defaultIsFromTemplate
	"Ground is pretty-much always scenery"
	^ true! !

!QCampusGround methodsFor: 'persist' stamp: 'bgf 9/21/2007 12:29'!
constructFrom:pStream
	super constructFrom: pStream.
	self children: (pStream readSequence:'children' default: self children).

	self children do: [ : kid |
		(kid isKindOf: QFractalTerrain) ifTrue: [
			kid material: (QMaterials groundMaterial: (kid material texture))
		].
	].! !

!QCampusGround methodsFor: 'initialize' stamp: 'bgf 4/17/2009 11:45'!
defaultIsAtomic
	"So the terrain layers are shown in the browser."
	^ false! !

!QCampusGround methodsFor: 'persist' stamp: 'bgf 3/7/2007 14:13'!
persistOn: pStream
	super persistOn: pStream.
	pStream storeSequence: 'children' value: self children default: #().! !
